from ...utils.vocabulary import BaseVocabulary


class FRVocabulary(BaseVocabulary):
    @property
    def STOPWORDS(self) -> tuple[str, ...]:
        return (
            "il",
            "elle",
            "ceci",
            "cela",
            "ce",
            "ces",
            "un",
            "une",
            "le",
            "la",
            "les",
            "des",
            "quelque",
            "quelques",
            "tout",
            "toute",
            "tous",
            "toutes",
            "aucun",
            "aucune",
            "tel",
            "telle",
            "ce qui suit",
            "quelque chose",
            "n'importe quoi",
            "tout",
            "rien",
            "quelqu'un",
            "n'importe qui",
            "personne",
        )

    @property
    def EPISTEMIC_KEYWORDS(self) -> dict[str, list[str]]:
        return {
            "future": [
                "prochain",
                "prochaine",
                "futur",
                "future",
                "sera",
                "attendu",
                "prévision",
                "prédiction",
                "à venir",
            ],
            "uncertainty": [
                "chance",
                "probabilité",
                "possibilité",
                "risque",
                "peut-être",
                "éventualité",
            ],
            "real_world": [
                "match",
                "saison",
                "météo",
                "marché",
                "action",
                "équipe",
                "jeu",
                "bourse",
            ],
        }

    @property
    def CODE_INDICATORS(self) -> tuple[str, ...]:
        return (
            "code",
            "script",
            "fonction",
            "programme",
            "algorithme",
            "extrait",
            "fichier",
            "implémentation",
            "syntaxe",
            "déboguer",
            "compiler",
            "exécution",
            "bibliothèque",
            "framework",
            "api",
            "classe",
            "méthode",
            "variable",
            "module",
            "paquet",
            "importer",
            "dépendance",
            "espace de noms",
            "interface",
            "dépôt",
            "repo",
            "git",
            "commit",
            "branche",
            "pull request",
            "pr",
            "fusion",
            "merge",
            "déployer",
            "build",
            "compilation",
            "test unitaire",
            "cas de test",
            "couverture",
            "lint",
            "refactoriser",
        )

    @property
    def PRONOUNS(self) -> tuple[str, ...]:
        return (
            "je",
            "nous",
            "tu",
            "vous",
            "ils",
            "elles",
            "il",
            "elle",
            "on",
            "me",
            "moi",
            "te",
            "toi",
            "lui",
            "leur",
            "eux",
            "mon",
            "ma",
            "mes",
            "notre",
            "nos",
            "ton",
            "ta",
            "tes",
            "votre",
            "vos",
            "son",
            "sa",
            "ses",
            "leur",
            "leurs",
            "le mien",
            "la mienne",
            "les miens",
            "les miennes",
            "le tien",
            "la tienne",
            "le sien",
            "la sienne",
            "le nôtre",
            "la nôtre",
            "le vôtre",
            "la vôtre",
            "le leur",
            "la leur",
        )

    @property
    def MODALS(self) -> tuple[str, ...]:
        return (
            "pouvoir",
            "peux",
            "peut",
            "pouvons",
            "pouvez",
            "peuvent",
            "pourrais",
            "pourrait",
            "devoir",
            "dois",
            "doit",
            "devons",
            "devez",
            "doivent",
            "devrais",
            "devrait",
            "vouloir",
            "veux",
            "veut",
            "voulons",
            "voulez",
            "veulent",
            "voudrais",
            "voudrait",
            "falloir",
            "faut",
            "il faut",
        )

    @property
    def ACTION_VERBS(self) -> tuple[str, ...]:
        return (
            "réduire",
            "augmenter",
            "améliorer",
            "optimiser",
            "perfectionner",
            "mettre à jour",
            "modifier",
            "refactoriser",
            "migrer",
            "corriger",
            "résoudre",
            "déboguer",
            "réparer",
            "diagnostiquer",
            "patcher",
            "créer",
            "générer",
            "construire",
            "faire",
            "produire",
            "concevoir",
            "développer",
            "implémenter",
            "analyser",
            "examiner",
            "réviser",
            "évaluer",
            "valider",
            "vérifier",
            "tester",
            "mesurer",
            "surveiller",
            "expliquer",
            "décrire",
            "clarifier",
            "définir",
            "documenter",
            "calculer",
            "déterminer",
            "trouver",
            "traiter",
            "comparer",
            "contraster",
            "différencier",
            "classifier",
            "catégoriser",
            "trier",
            "organiser",
            "regrouper",
            "extraire",
            "transformer",
            "charger",
            "agréger",
            "filtrer",
            "déployer",
            "libérer",
            "annuler",
            "mettre à l'échelle",
            "provisionner",
            "souligner",
            "identifier",
            "montrer",
            "démontrer",
            "afficher",
        )

    def __init__(self):
        super().__init__()

    @property
    def QUANTIFIER_WORDS(self) -> tuple[str, ...]:
        return ("tous", "toutes", "tout", "chaque", "entier", "complet")

    @property
    def DEMONSTRATIVES(self) -> list[str]:
        return ["ce", "cet", "cette", "ces"]

    @property
    def COMPOUND_PHRASES(self) -> dict[str, str]:
        return {
            "support client": "TICKET",
            "ticket de support": "TICKET",
            "message email": "EMAIL",
            "transcription de chat": "TRANSCRIPT",
            "appel téléphonique": "CALL",
            "code source": "CODE",
            "plan d'affaires": "PLAN",
            "description du produit": "DESCRIPTION",
        }

    @property
    def domain_candidates(self) -> dict[str, list[str]]:
        return {
            "CODE": ["bug", "erreur", "sécurité", "performance"],
            "ENTITIES": [
                "noms",
                "dates",
                "montants",
                "adresses",
                "emails",
                "courriels",
                "téléphones",
            ],
            "QA": [
                "vérification",
                "politique",
                "compétences relationnelles",
                "soft skills",
                "précision",
                "exactitude",
                "conformité",
                "compliance",
                "divulgations",
            ],
            "SUPPORT": [
                "problème",
                "question",
                "sentiment",
                "actions",
                "urgence",
                "priorité",
                "ticket",
                "dossier",
                "client",
                "support",
                "appel",
                "réclamation",
                "csr",
                "agent",
                "interlocuteur",
                "chat",
                "demande",
                "dépannage",
            ],
            "TECHNICAL": [
                "bug",
                "erreur",
                "stacktrace",
                "exception",
                "api",
                "serveur",
                "journal",
                "débogage",
                "trace",
                "crash",
                "déploiement",
                "backend",
            ],
            "DOCUMENT": [
                "document",
                "article",
                "manuel",
                "guide",
                "doc",
                "rédaction",
                "transcription",
                "notes",
                "résumé",
                "instructions",
            ],
            "BUSINESS": [
                "rapport",
                "briefing",
                "analyse",
                "exécutif",
                "direction",
                "tableau de bord",
                "kpi",
                "roi",
                "entonnoir",
                "trimestriel",
                "présentation",
            ],
            "LEGAL": [
                "contrat",
                "politique",
                "conformité",
                "rgpd",
                "clause",
                "avocat",
                "accord",
                "conditions",
                "confidentialité",
            ],
            "FINANCE": [
                "facture",
                "facturation",
                "paiement",
                "transaction",
                "remboursement",
                "dépense",
                "solde",
                "relevé",
                "montant",
            ],
            "SECURITY": [
                "violation",
                "risque",
                "menace",
                "alerte",
                "malware",
                "phishing",
                "permissions",
                "contrôle d'accès",
                "audit",
            ],
            "MEDICAL": [
                "patient",
                "diagnostic",
                "prescription",
                "clinique",
                "dossier médical",
                "symptômes",
                "traitement",
                "médecin",
            ],
            "SALES": [
                "lead",
                "crm",
                "opportunité",
                "pipeline",
                "gestionnaire de compte",
                "prospect",
                "affaire",
                "devis",
            ],
            "EDUCATION": [
                "leçon",
                "programme",
                "enseignant",
                "étudiant",
                "formation",
                "cours",
                "apprentissage",
            ],
        }

    @property
    def rank_triggers(self) -> set[str]:
        return {
            "classer",
            "ranger",
            "trier",
            "ordonner",
            "ordre",
            "ordonner par",
            "trier par",
            "classer par",
            "prioriser",
            "haut",
            "supérieur",
            "premier",
            "top",
            "bas",
            "inférieur",
            "dernier",
            "le plus haut",
            "le plus élevé",
            "maximum",
            "le plus bas",
            "minimum",
            "meilleur",
            "meilleurs",
            "meilleures",
            "pire",
            "pires",
        }

    @property
    def CONCEPT_INDICATORS(self) -> list[str]:
        """Optional - can have sensible default"""
        return ["concept de", "idée de", "notion de", "principe de"]

    @property
    def MEETING_WORDS(self) -> tuple[str, ...]:
        """Optional - can have empty default"""
        return ("réunion", "conférence", "rencontre", "séance")

    @property
    def PROPOSAL_WORDS(self) -> list[str]:
        """Optional - can have empty default"""
        return ["proposition", "propositon", "offre", "suggestion", "idée"]

    @property
    def ARTICLES(self) -> list[str]:
        """Optional - can have empty default"""
        return ["le", "la", "les", "l'", "un", "une", "des"]

    @property
    def REQ_TOKENS(self) -> dict[str, list[str]]:
        return {
            "ANALYZE": [
                "analyser",
                "réviser",
                "examiner",
                "évaluer",
                "inspecter",
                "vérifier",
                "auditer",
                "enquêter",
                "étudier",
            ],
            "MATCH": [
                "faire correspondre",
                "comparer",
                "aligner",
                "mapper",
                "corréler",
                "apparier",
                "croiser",
                "rapprocher",
            ],
            "EXTRACT": [
                "extraire",
                "retirer",
                "identifier",
                "trouver",
                "localiser",
                "obtenir",
                "récupérer",
                "retourner",
                "inclure",
                "sélectionner",
                "chercher",
            ],
            "SELECT": [
                "sélectionner",
                "choisir",
                "filtrer",
                "identifier les correspondances",
                "opter",
                "élire",
            ],
            "GENERATE": [
                "générer",
                "créer",
                "écrire",
                "rédiger",
                "composer",
                "produire",
                "construire",
                "développer",
                "concevoir",
                "élaborer",
                "faire",
                "nommer",
                "suggérer",
                "formuler",
                "former",
                "proposer",
                "transcrire",
            ],
            "SUMMARIZE": [
                "résumer",
                "condenser",
                "synthétiser",
                "récapituler",
                "abréger",
            ],
            "TRANSFORM": [
                "convertir",
                "transformer",
                "changer",
                "réécrire",
                "traduire",
                "compléter",
                "modifier",
                "adapter",
                "ajuster",
                "reformuler",
                "altérer",
                "éditer",
                "ajouter",
                "paraphraser",
                "remplir",
                "supprimer",
                "enlever",
                "remplacer",
                "inverser",
            ],
            "EXPLAIN": [
                "expliquer",
                "décrire",
                "clarifier",
                "élaborer",
                "détailler",
                "exposer",
                "illustrer",
                "exprimer",
                "raconter",
                "discuter",
                "définir",
                "parler de",
                "dis-moi",
                "parle-moi de",
            ],
            "COMPARE": [
                "comparer",
                "contraster",
                "versus",
                "vs",
                "différence entre",
                "différencier",
                "distinguer",
            ],
            "CLASSIFY": [
                "classifier",
                "catégoriser",
                "trier",
                "regrouper",
                "étiqueter",
                "organiser",
                "ranger",
                "segmenter",
            ],
            "DEBUG": [
                "déboguer",
                "dépanner",
                "diagnostiquer",
                "corriger un bug",
                "trouver un bug",
                "traquer",
                "identifier le problème",
            ],
            "OPTIMIZE": [
                "optimiser",
                "améliorer",
                "perfectionner",
                "refactoriser",
                "accélérer",
                "simplifier",
                "maximiser",
                "minimiser",
                "réduire",
                "augmenter",
            ],
            "VALIDATE": [
                "valider",
                "vérifier",
                "contrôler",
                "confirmer",
                "tester",
                "assurer",
                "certifier",
                "authentifier",
            ],
            "SEARCH": [
                "chercher",
                "rechercher",
                "trouver",
                "localiser",
                "consulter",
                "découvrir",
            ],
            "RANK": [
                "prioriser",
                "ordonner",
                "classer par",
                "noter",
                "évaluer",
                "hiérarchiser",
            ],
            "PREDICT": [
                "prédire",
                "prévoir",
                "estimer",
                "projeter",
                "anticiper",
                "extrapoler",
                "pronostiquer",
            ],
            "FORMAT": [
                "formater",
                "structurer",
                "organiser",
                "disposer",
                "mettre en forme",
            ],
            "DETECT": [
                "détecter",
                "repérer",
                "découvrir",
                "percevoir",
                "reconnaître",
                "remarquer",
                "identifier",
            ],
            "CALCULATE": [
                "calculer",
                "computer",
                "déterminer",
                "quantifier",
                "mesurer",
                "compter",
                "additionner",
                "totaliser",
            ],
            "AGGREGATE": [
                "agréger",
                "regrouper",
                "combiner",
                "consolider",
                "rassembler",
                "fusionner",
                "compiler",
                "collecter",
                "réunir",
            ],
            "DETERMINE": [
                "déterminer",
                "décider",
                "évaluer",
                "conclure",
                "établir",
                "choisir",
                "définir",
            ],
            "ROUTE": [
                "router",
                "attribuer",
                "diriger",
                "transférer",
                "envoyer à",
                "déléguer",
                "affecter",
            ],
            "EXECUTE": [
                "utiliser",
                "appliquer",
                "implémenter",
                "exécuter",
                "réaliser",
                "employer",
                "mettre en œuvre",
            ],
            "LIST": [
                "lister",
                "énumérer",
                "détailler",
                "dresser la liste",
                "répertorier",
            ],
        }

    @property
    def NOISE_VERBS(self) -> set[str]:
        return {
            "être",
            "avoir",
            "faire",
            "pouvoir",
            "devoir",
            "vouloir",
            "aller",
            "venir",
            "falloir",
            "rester",
            "vivre",
            "habiter",
            "continuer",
            "demeurer",
            "suivre",
            "baser",
            "avoir besoin",
            "commencer",
            "devenir",
            "sembler",
            "paraître",
            "apparaître",
            "montrer",
            "voir",
            "savoir",
            "penser",
            "sentir",
            "dire",
            "parler",
            "appeler",
        }

    @property
    def CONTEXT_FILTERS(self) -> dict[str, list[str]]:
        return {
            "donner": ["donné", "donnant", "donne-moi", "donnez-moi"],
            "baser": ["basé sur", "en se basant sur"],
            "utiliser": ["utile", "utilisé pour", "utilisable"],
            "continuer": ["continu", "continuer à", "continuer de"],
            "demeurer": ["demeur", "demeurer à", "demeurer de"],
            "suivre": ["suivant", "ci-dessous", "comme suit"],
            "avoir besoin": ["besoin", "nécessité", "nécessité de"],
            "commencer": ["commenc", "commencer à", "commencer de"],
            "devenir": ["deven", "devenir à", "devenir de"],
            "sembler": ["semble", "sembler à", "sembler de"],
            "paraître": ["paraît", "paraître à", "paraître de"],
            "apparaître": ["apparaît", "apparaître à", "apparaître de"],
            "montrer": ["montre", "montrer à", "montrer de"],
            "voir": ["voit", "voir à", "voir de"],
            "savoir": ["sait", "savoir à", "savoir de"],
            "penser": ["pense", "penser à", "penser de"],
            "sentir": ["sent", "sentir à", "sentir de"],
            "dire": ["dit", "dire à", "dire de"],
            "parler": ["parle", "parler à", "parler de"],
            "appeler": ["appelle", "appeler à", "appeler de"],
        }

    @property
    def TARGET_TOKENS(self) -> dict[str, list[str]]:
        return {
            "CODE": [
                "code",
                "script",
                "programme",
                "fonction",
                "méthode",
                "classe",
                "extrait",
            ],
            "DATA": [
                "données",
                "dataset",
                "jeu de données",
                "base de données",
                "feuille de calcul",
                "tableau",
                "csv",
            ],
            "QUERY": [
                "requête",
                "query",
                "sql",
                "requête sql",
                "requête de base de données",
            ],
            "ENDPOINT": [
                "endpoint",
                "api",
                "point d'accès",
                "service rest",
            ],
            "COMPONENT": [
                "composant",
                "module",
                "paquet",
                "bibliothèque",
                "librairie",
            ],
            "SYSTEM": [
                "système",
                "application",
                "appli",
                "logiciel",
                "plateforme",
            ],
            "TEST": [
                "test",
                "test unitaire",
                "cas de test",
                "suite de tests",
            ],
            "LOG": [
                "log",
                "logs",
                "fichier de log",
                "journal",
                "historique",
            ],
            "RECORD": [
                "enregistrement",
                "entrée",
                "ligne",
                "élément",
            ],
            "STRATEGY": [
                "stratégie",
                "approche",
                "plan",
                "méthodologie",
            ],
            "NBA_CATALOG": [
                "nba",
                "nbas",
                "prochaine meilleure action",
                "actions prédéfinies",
                "actions possibles",
                "actions disponibles",
            ],
            "DOCUMENT": [
                "document",
                "doc",
                "fichier",
                "rapport",
                "article",
            ],
            "EMAIL": [
                "email",
                "courriel",
                "message",
                "correspondance",
            ],
            "REPORT": [
                "rapport",
                "analyse",
                "conclusions",
                "compte rendu",
            ],
            "TICKET": [
                "ticket",
                "incident",
                "cas",
                "demande",
            ],
            "TRANSCRIPT": [
                "transcription",
                "conversation",
                "dialogue",
                "historique de chat",
            ],
            "FEEDBACK": [
                "feedback",
                "retour",
                "commentaire",
                "évaluation",
                "critique",
                "avis",
            ],
            "COMMENT": [
                "commentaire",
                "remarque",
                "note",
                "observation",
            ],
            "CUSTOMER_INTENT": [
                "intention du client",
                "besoin du client",
                "demande du client",
                "objectif du client",
                "problème du client",
            ],
            "COMPLAINT": [
                "plainte",
                "réclamation",
                "problème",
                "grief",
            ],
            "REQUEST": [
                "demande",
                "requête",
                "sollicitation",
            ],
            "INQUIRY": [
                "demande de renseignement",
                "question",
                "interrogation",
            ],
            "INTERACTION": [
                "interaction",
                "échange",
                "contact",
            ],
            "CALL": [
                "appel",
                "appel téléphonique",
            ],
            "DESCRIPTION": [
                "description",
                "détail",
            ],
            "SUMMARY": [
                "résumé",
                "sommaire",
                "synthèse",
            ],
            "PLAN": [
                "plan",
                "planification",
                "projet",
            ],
            "POST": [
                "post",
                "publication",
                "article",
            ],
            "PRESS_RELEASE": [
                "communiqué de presse",
                "annonce",
            ],
            "INTRODUCTION": [
                "introduction",
                "présentation",
            ],
            "CAPTION": [
                "légende",
                "sous-titre",
            ],
            "QUESTIONS": [
                "questions",
                "questionnaire",
                "faq",
            ],
            "RESPONSE": [
                "réponse",
                "réplique",
                "modèle de réponse",
            ],
            "LOGS": [
                "logs",
                "journaux",
                "historiques",
            ],
            "CORRELATION": [
                "corrélation",
                "corrélations",
                "relation",
                "relations",
            ],
            "TRADEOFF": [
                "compromis",
                "arbitrage",
            ],
            "PARADIGM": [
                "paradigme",
                "paradigmes",
                "modèle",
            ],
            "PAIN_POINTS": [
                "points de douleur",
                "problèmes",
                "difficultés",
                "irritants",
            ],
            "PATTERN": [
                "motif",
                "modèle",
                "tendance",
                "tendances",
                "pattern",
            ],
            "CHURN": [
                "churn",
                "attrition",
                "désabonnement",
                "résiliation",
            ],
            "FEATURES": [
                "fonctionnalités",
                "caractéristiques",
                "features",
            ],
            "METRICS": [
                "revenus",
                "métriques",
                "statistiques",
                "chiffres",
                "indicateurs",
            ],
            "REGIONS": [
                "régions",
                "zones",
                "emplacements",
                "territoires",
            ],
            "ITEMS": [
                "éléments",
                "choses",
                "liste",
                "options",
                "choix",
            ],
            "CONCEPT": [
                "concept",
                "idée",
                "notion",
                "principe",
                "théorie",
            ],
            "PROCEDURE": [
                "procédure",
                "processus",
                "étapes",
                "méthode",
                "technique",
            ],
            "FACT": [
                "fait",
                "faits",
                "information",
                "informations",
                "détails",
            ],
            "ANSWER": [
                "réponse",
                "solution",
            ],
        }

    @property
    def EXTRACT_FIELDS(self) -> tuple[str, ...]:
        return (
            "ISSUE",
            "SENTIMENT",
            "ACTIONS",
            "NEXT_STEPS",
            "URGENCY",
            "PRIORITY",
            "NAMES",
            "DATES",
            "AMOUNTS",
            "EMAILS",
            "PHONES",
            "ADDRESSES",
            "BUGS",
            "SECURITY",
            "PERFORMANCE",
            "STYLE",
            "ERRORS",
            "WARNINGS",
            "KEYWORDS",
            "TOPICS",
            "ENTITIES",
            "FACTS",
            "DECISIONS",
            "DEADLINES",
            "REQUIREMENTS",
            "FEATURES",
            "PROBLEMS",
            "SOLUTIONS",
            "RISKS",
            "METRICS",
            "KPI",
            "SCORES",
            "RATINGS",
            "FEEDBACK",
            "COMPLAINTS",
            "OWNERS",
            "ASSIGNEES",
            "STAKEHOLDERS",
            "PARTICIPANTS",
            "TIMESTAMPS",
            "DURATIONS",
            "FREQUENCIES",
            "QUANTITIES",
            "CATEGORIES",
            "TAGS",
            "LABELS",
            "STATUS",
            "TYPE",
            "CUSTOMER_INTENT",
            "RELEVANCE_SCORE",
            "NBA_ID",
            "MATCH_CONFIDENCE",
            "SEMANTIC_SIMILARITY",
            "THRESHOLD",
        )

    @property
    def OUTPUT_FORMATS(self) -> dict[str, list[str]]:
        return {
            "JSON": ["json", "format json"],
            "MARKDOWN": ["markdown", "md"],
            "TABLE": ["tableau", "tabulaire"],
            "LIST": ["liste", "puces", "points"],
            "PLAIN": ["texte brut", "texte simple"],
            "CSV": ["csv", "séparé par des virgules"],
        }

    @property
    def IMPERATIVE_PATTERNS(self) -> list[tuple[list[str], str, str]]:
        return [
            (["liste", "lister", "énumère", "énumérer"], "LIST", "ITEMS"),
            (["nomme", "nommer", "identifie", "identifier"], "GENERATE", "ITEMS"),
            (
                ["donne", "donner", "fournis", "fournir", "suggère", "suggérer"],
                "GENERATE",
                "ITEMS",
            ),
            (
                [
                    "dis",
                    "dire",
                    "explique",
                    "expliquer",
                    "décris",
                    "décrire",
                    "clarifie",
                    "clarifier",
                    "illustre",
                    "illustrer",
                ],
                "EXPLAIN",
                "CONCEPT",
            ),
        ]

    @property
    def QUESTION_WORDS(self) -> list[str]:
        return [
            "que",
            "qu'est-ce que",
            "qui",
            "où",
            "quand",
            "pourquoi",
            "comment",
            "quel",
            "quelle",
            "quels",
            "quelles",
            "lequel",
            "laquelle",
        ]



from abc import ABC, abstractmethod

from clm_core import CLMOutput


class BasePromptEncoder(ABC):
    @abstractmethod
    def compress(self, prompt: str, verbose: bool = False) -> CLMOutput:
        raise NotImplementedError

    @abstractmethod
    def compress_batch(
        self, prompts: list[str], verbose: bool = False
    ) -> list[CLMOutput]:
        raise NotImplementedError



import re
from spacy.language import Language

from clm_core.utils.parser_rules import BaseRules
from clm_core.utils.vocabulary import BaseVocabulary

from clm_core import SysPromptConfig
from .analyzers.attribute_parser import AttributeParser
from clm_core.components.intent_detector_v2 import IntentDetectorV2 as IntentDetector
from clm_core.components.target_extractor import TargetExtractor
from .base_encoder import BasePromptEncoder
from .tokenizer import CLLMTokenizer
from clm_core.types import CLMOutput

COMPONENT = "TASK_PROMPT"


class TaskPromptEncoder(BasePromptEncoder):
    def __init__(
        self,
        *,
        nlp: Language,
        config: SysPromptConfig = SysPromptConfig(),
        vocab: BaseVocabulary,
        rules: BaseRules,
    ):
        """
        Initialize encode

        Args:
            nlp: spaCy model to use (en_core_web_sm, en_core_web_md, en_core_web_lg)
        """

        self.nlp: Language = nlp
        self._config = config
        self._vocab = vocab
        self._rules = rules

        self.intent_detector = IntentDetector(self.nlp, vocab=self._vocab)
        self.target_extractor = TargetExtractor(
            self.nlp, vocab=self._vocab, rules=self._rules
        )
        self.attribute_parser = AttributeParser(
            nlp=self.nlp, config=config, vocab=self._vocab, rules=rules
        )
        self.tokenizer = CLLMTokenizer()

    def compress(self, prompt: str, verbose: bool = False) -> CLMOutput:
        """
        Compress a natural language prompt into CLLM format

        Args:
            prompt: Natural language prompt to compress
            verbose: Print detailed compression steps

        Returns:
            CompressionResult with compressed format and metadata
        """
        if verbose:
            print(f"\n{'=' * 60}")
            print(f"Compressing: {prompt}")
            print(f"{'=' * 60}")

        intent = self.intent_detector.detect(text=prompt)
        if verbose:
            print(f"\n1. Intents detected: {intent.token}")

        target = self.target_extractor.extract(prompt)
        if verbose:
            print(f"2. Targets detected: {target.token}")

        extractions = self.attribute_parser.parse_extraction_fields(prompt)
        quantifiers = self.attribute_parser.extract_quantifier(prompt)
        specifications = self.attribute_parser.extract_specifications(prompt)
        if verbose and extractions:
            print(f"3. Extraction fields: {extractions.fields}")
            print(f"3.1 Quantifiers field: {quantifiers}")

        contexts = self.attribute_parser.parse_contexts(prompt)
        if verbose and contexts:
            print(f"4. Contexts: {[(c.aspect, c.value) for c in contexts]}")

        output_format = self.attribute_parser.parse_output_format(prompt)
        if verbose and output_format:
            print(f"5. Output format: {output_format.format_type}")

        compressed = self.tokenizer.build_sequence(
            intent=intent,
            contexts=contexts,
            target=target,
            output_format=output_format,
            extractions=extractions,
            quantifier=quantifiers,
            specifications=specifications,
        )

        doc = self.nlp(prompt)
        verbs = [token.lemma_ for token in doc if token.pos_ == "VERB"]

        if verbose:
            print(f"\n{'=' * 60}")
            print(f"Compressed: {compressed}")
            print(f"{'=' * 60}\n")

        return CLMOutput(
            original=prompt,
            compressed=compressed,
            component=COMPONENT,
            metadata={
                "original_length": len(prompt),
                "compressed_length": len(compressed),
                "num_intents": 1 if intent.token else 0,
                "num_targets": 1 if target else 0,
                "verbs": verbs,
                "intents": intent.model_dump(),
                "target": target,
                "extractions": extractions,
                "contexts": contexts,
                "output_format": output_format,
                "noun_chunks": [chunk.text for chunk in doc.noun_chunks],
                "language": "en",
                "has_numbers": bool(re.search(r"\d", prompt)),
                "has_urls": bool(re.search(r"https?://", prompt)),
                "has_code_indicators": any(
                    word in prompt.lower()
                    for word in ["python", "javascript", "function", "class"]
                ),
            },
        )

    def compress_batch(
        self, prompts: list[str], verbose: bool = False
    ) -> list[CLMOutput]:
        """Compress multiple prompts"""
        results = []
        for i, prompt in enumerate(prompts, 1):
            if verbose:
                print(f"\n[{i}/{len(prompts)}]")
            result = self.compress(prompt, verbose=verbose)
            results.append(result)
        return results



from enum import Enum
from typing import Any
import langdetect


class DataTypes(Enum):
    """Data types to start the CLM encoding process"""

    SYSTEM_PROMPT = 0
    TRANSCRIPT = 1
    STRUCTURED_DATA = 2
    UNK = -1


class DataClassifier:
    """
    Classifies input data into CLM-compatible types.

    Expected inputs:
    - System Prompt: str (instruction text)
    - Transcript: str (conversation with speakers)
    - Structured Data: list[dict] or dict (catalog items)
    """

    def __init__(self) -> None:
        self._system_prompt_patterns: list[str] = [
            "your task is",
            "you are a",
            "your goal is to",
            "you must",
            "you should",
            "your role is",
            "instructions:",
            "please analyze",
            "please extract",
            "the following",  # Often in system prompts
        ]

        self._transcript_patterns: dict[str, tuple[str, ...]] = {
            "en": (
                "agent:",
                "customer:",
                "user:",
                "assistant:",
                "caller:",
                "representative:",
            ),
            "es": (
                "agente:",
                "cliente:",
                "usuario:",
                "asistente:",
                "llamante:",
                "representante:",
            ),
            "pt": (
                "agente:",
                "cliente:",
                "utilizador:",
                "assistente:",
                "usuario:",
                "representante:",
            ),
            "fr": (
                "agent :",
                "client :",
                "utilisateur :",
                "assistant :",
                "appelant :",
                "représentant :",
            ),
        }

    def classifier(self, *, input_: Any) -> DataTypes:
        """
        Classify input data type.

        Args:
            input_: Any input data to classify

        Returns:
            DataTypes enum indicating the classified type

        Examples:
            >>> classifier.classify(input_="Your task is to analyze...")
            DataTypes.SYSTEM_PROMPT

            >>> classifier.classify(input_="Agent: Hello\\nCustomer: Hi")
            DataTypes.TRANSCRIPT

            >>> classifier.classify(input_=[{"id": "NBA-001", ...}, ...])
            DataTypes.STRUCTURED_DATA
        """
        if input_ is None or (isinstance(input_, str) and not input_.strip()):
            return DataTypes.UNK

        if self._is_structured_data(input_):
            return DataTypes.STRUCTURED_DATA

        if not isinstance(input_, str):
            return DataTypes.UNK

        normalized = input_.lower().strip()

        if self._is_transcript(normalized):
            return DataTypes.TRANSCRIPT

        if self._is_system_prompt(normalized):
            return DataTypes.SYSTEM_PROMPT

        return DataTypes.UNK

    @staticmethod
    def _is_structured_data(input_: Any) -> bool:
        """
        Check if input is structured data (catalog).

        Structured data is:
        - list of dicts (catalog items)
        - dict with known catalog structure

        NOT:
        - strings (even though they're iterable!)
        - empty collections
        """

        if isinstance(input_, str):
            return False

        if isinstance(input_, list):
            if not input_:
                return False
            return isinstance(input_[0], dict)

        if isinstance(input_, dict):
            return True

        return False

    def _is_transcript(self, normalized_text: str) -> bool:
        """
        Check if text is a thread_encoder (conversation).

        Transcripts have:
        - Speaker labels (Agent:, Customer:, etc.)
        - Multiple exchanges (at least 2 speaker occurrences)
        """
        lang = langdetect.detect(normalized_text[:200])
        patterns = self._transcript_patterns[lang]
        speaker_count = sum(
            normalized_text.count(pattern) for pattern in patterns[:250]
        )

        return speaker_count >= 2

    def _is_system_prompt(self, normalized_text: str) -> bool:
        """
        Check if text is a system prompt (instructions).

        System prompts typically:
        - Give instructions or define tasks
        - Are longer than a single sentence
        - Contain imperative language
        """
        # Check for system prompt patterns
        has_prompt_pattern = any(
            pattern in normalized_text for pattern in self._system_prompt_patterns
        )

        return has_prompt_pattern



import re
from typing import Optional
from datetime import datetime
from pydantic import BaseModel, Field, ConfigDict
from spacy.tokens import Doc

from clm_core import CLMOutput
from clm_core.components.sys_prompt import Intent, Target


class Turn(BaseModel):
    """Single turn in conversation"""

    speaker: str = Field(..., description="agent, customer, system")
    text: str = Field(..., description="text of the turn")
    timestamp: Optional[float] = None
    intent: Optional[Intent] = None
    targets: list[Target] = Field(default_factory=list)
    sentiment: Optional[str] = None
    entities: dict = Field(default_factory=dict)
    doc: Optional[Doc] = None

    model_config = ConfigDict(arbitrary_types_allowed=True)


class CallInfo(BaseModel):
    """Call metadata"""

    call_id: str
    type: str = Field(..., description="type of call: SUPPORT, SALES, BILLING, etc.")
    channel: str = Field(..., description="channel of call: VOICE, CHAT, EMAIL")
    duration: int = Field(..., description="duration of call in minutes")
    agent: Optional[str] = None
    timestamp: Optional[datetime] = None


class Issue(BaseModel):
    """Customer issue"""

    type: str = Field(
        ..., description="type of issue: INTERNET_OUTAGE, BILLING_DISPUTE, etc."
    )
    disputed_amounts: list[str] = Field(default_factory=list)
    cause: Optional[str] = None
    plan_change: Optional[str] = None
    severity: Optional[str] = Field(
        default="LOW", description="severity of issue: LOW, MEDIUM, HIGH, CRITICAL"
    )
    frequency: Optional[str] = Field(
        default=None, description="frequency of issue: ONCE, DAILY, etc."
    )
    duration: Optional[str] = Field(
        default=None, description="duration of issue: '3d', '1week'"
    )
    pattern: Optional[str] = Field(
        default=None, description="pattern of issue: '9am+1pm+6pm'"
    )
    impact: Optional[str] = Field(
        default=None, description="impact of issue: WORK_FROM_HOME, etc."
    )
    attributes: dict = Field(default_factory=dict)


class Action(BaseModel):
    """Action taken by agent"""

    type: str = Field(..., description="type of action: TROUBLESHOOT, ESCALATE, etc.")
    step: Optional[str] = None
    result: Optional[str] = Field(
        default="PENDING", description="result of action: SUCCESS, FAILED, TEMP_FIX"
    )
    timestamp: Optional[float] = None
    attributes: dict = Field(default_factory=dict)
    amount: Optional[str] = None
    payment_method: Optional[str] = None


class Resolution(BaseModel):
    """How conversation resolved"""

    type: str = Field(
        default="UNKNOWN",
        description="type of resolution: RESOLVED, PENDING, ESCALATED, etc.",
    )
    status: Optional[str] = None
    timeline: Optional[str] = Field(
        default=None, description="timeline of resolution: '24h', '3-5_days'"
    )
    next_steps: Optional[str] = None
    ticket_id: Optional[str] = None


class SentimentTrajectory(BaseModel):
    """Sentiment across conversation"""

    start: Optional[str] = Field(
        default="NEUTRAL",
        description="start sentiment: 'NEUTRAL', 'POSITIVE', 'NEGATIVE'",
    )
    end: Optional[str] = Field(
        default="NEUTRAL",
        description="end sentiment: 'NEUTRAL', 'POSITIVE', 'NEGATIVE'",
    )
    turning_points: list[tuple[int, str]] = Field(
        default_factory=list, description="turning points: [(turn_number, sentiment)]"
    )
    intensity: Optional[float] = None


class CustomerProfile(BaseModel):
    """Customer information"""

    account: Optional[str] = None
    tier: Optional[str] = Field(
        default=None,
        description="customer tier: 'FREE', 'BASIC', 'PREMIUM', 'ENTERPRISE'",
    )
    tenure: Optional[str] = Field(default=None, description="customer tenure: '5yr'")
    lifetime_value: Optional[float] = None
    churn_risk: Optional[str] = Field(
        default=None, description="churn risk: 'LOW', 'MEDIUM', 'HIGH'"
    )
    attributes: Optional[dict] = None
    name: Optional[str] = Field(default=None, description="Customer's name")
    email: Optional[str] = Field(default=None, description="Customer's email")


class ResolutionState(BaseModel):
    """Enhanced resolution state with granularity"""

    type: str = Field(
        default="UNKNOWN",
        description="FULLY_RESOLVED, PARTIALLY_RESOLVED, PENDING, ESCALATED, UNRESOLVED",
    )
    completeness: Optional[str] = Field(
        default=None,
        description="FULL, PARTIAL, NONE - how much of issue was addressed",
    )
    customer_satisfaction: Optional[str] = Field(
        default=None, description="SATISFIED, NEUTRAL, DISSATISFIED"
    )
    follow_up_needed: bool = Field(
        default=False, description="Whether follow-up is required"
    )
    follow_up_reason: Optional[str] = Field(
        default=None,
        description="PENDING_ACTION, VERIFICATION_NEEDED, SCHEDULED_CALLBACK",
    )


class RefundReference(BaseModel):
    """Case-dependent refund information for billing/refund cases"""

    reference_number: Optional[str] = Field(
        default=None, description="Refund reference ID"
    )
    amount: Optional[str] = Field(default=None, description="Refund amount like $14.99")
    method: Optional[str] = Field(
        default=None, description="CARD_CREDIT, ACCOUNT_CREDIT, CHECK, PAYPAL"
    )
    status: Optional[str] = Field(
        default=None, description="INITIATED, PROCESSING, COMPLETED, PENDING_APPROVAL"
    )
    timeline: Optional[str] = Field(
        default=None, description="Expected timeline: 3-5_DAYS, 24h, IMMEDIATE"
    )
    original_transaction_id: Optional[str] = Field(
        default=None, description="Original transaction being refunded"
    )


class TimelineEvent(BaseModel):
    """Single event in conversation timeline"""

    event_type: str = Field(
        ..., description="ISSUE_RAISED, ACTION_TAKEN, RESOLUTION_PROPOSED, etc."
    )
    description: Optional[str] = Field(
        default=None, description="Brief description of event"
    )
    turn_index: int = Field(..., description="Turn index where event occurred")
    timestamp: Optional[float] = Field(
        default=None, description="Relative timestamp if available"
    )
    actor: str = Field(
        default="agent", description="Who triggered: agent, customer, system"
    )


class ConversationTimeline(BaseModel):
    """Timeline of key events in conversation"""

    events: list[TimelineEvent] = Field(
        default_factory=list, description="Ordered list of events"
    )
    first_issue_turn: Optional[int] = Field(
        default=None, description="Turn index when issue was first raised"
    )
    first_resolution_turn: Optional[int] = Field(
        default=None, description="Turn index when resolution was first proposed"
    )
    time_to_first_action: Optional[int] = Field(
        default=None, description="Turns between issue and first action"
    )
    time_to_resolution: Optional[int] = Field(
        default=None, description="Turns between issue and resolution"
    )


class PromiseCommitment(BaseModel):
    """Agent promise/commitment to customer"""

    type: str = Field(
        ...,
        description="CALLBACK, FOLLOW_UP_EMAIL, TECHNICIAN_VISIT, CREDIT_PROMISE, etc.",
    )
    description: str = Field(..., description="What was promised")
    timeline: Optional[str] = Field(
        default=None, description="When promised: 24h, MONDAY, 3-5_DAYS"
    )
    amount: Optional[str] = Field(
        default=None, description="Amount if applicable (credit/refund)"
    )
    turn_index: int = Field(..., description="Turn where promise was made")
    confidence: float = Field(default=0.8, description="Detection confidence 0.0-1.0")


class MonetaryAmount(BaseModel):
    """Monetary amount extracted from a conversation turn"""

    amount: str = Field(..., description="Amount string e.g. '$2.99'")
    reason: Optional[str] = Field(
        default=None,
        description="Reason category: REFUND, CHARGE, FEE, CREDIT, EXTRA_CHARGE, DUPLICATE_CHARGE, DISCOUNT",
    )
    speaker: str = Field(
        ..., description="Who mentioned the amount: customer, agent, system"
    )
    turn_index: int = Field(..., description="Index of the turn where amount appeared")


class TranscriptAnalysis(BaseModel):
    """Complete thread_encoder analysis"""

    call_info: CallInfo = Field(..., description="Transcript metadata information")
    customer: CustomerProfile = Field(..., description="Customer profile information")
    turns: list[Turn] = Field(..., description="Turns information (Agent/Customer)")
    issues: list[Issue] = Field(..., description="Issues information")
    actions: list[Action] = Field(
        ..., description="Actions performed by the agent during the conversation"
    )
    resolution: Resolution = Field(
        ..., description="Resolution detected during the conversation"
    )
    sentiment_trajectory: SentimentTrajectory = Field(
        ..., description="Sentiment trajectory information"
    )
    resolution_state: Optional[ResolutionState] = Field(
        default=None, description="Enhanced resolution state"
    )
    refund_reference: Optional[RefundReference] = Field(
        default=None, description="Refund details (only for billing/refund cases)"
    )
    timeline: Optional[ConversationTimeline] = Field(
        default=None, description="Timeline of key events"
    )
    promises: list[PromiseCommitment] = Field(
        default_factory=list, description="Agent promises/commitments"
    )

    # v2 schema fields — amounts and redacted fields
    amounts: list[MonetaryAmount] = Field(
        default_factory=list,
        description="All monetary amounts extracted from all turns with their reason",
    )
    redacted_fields: list[str] = Field(
        default_factory=list,
        description="Detected redacted field types: EMAIL_REDACTED, PHONE_REDACTED, etc.",
    )

    domain: Optional[str] = Field(
        default=None,
        description="Domain classification: BILLING, AUTHENTICATION, BOOKINGS, API, PERFORMANCE, etc.",
    )
    service: Optional[str] = Field(
        default=None,
        description="Service area: SUBSCRIPTION, HOST_STAY, PAYMENT, DASHBOARD, EXPORTS, etc.",
    )
    customer_intent: Optional[str] = Field(
        default=None,
        description="Primary customer intent derived from customer utterances",
    )
    secondary_intent: Optional[str] = Field(
        default=None,
        description="Optional secondary customer intent",
    )
    trigger_cause: Optional[str] = Field(
        default=None,
        description="Why the issue happened: the trigger/cause that prompted the customer to contact support (e.g. FIELD_LOCKED, MISSING_DELIVERY)",
    )
    context_provided: list[str] = Field(
        default_factory=list,
        description="Context provided by customer without PII: EMAIL_PROVIDED, BOOKING_ID_PROVIDED, etc.",
    )
    system_actions: list[str] = Field(
        default_factory=list,
        description="Automated system-level events: PAYMENT_RETRY_DETECTED, etc.",
    )

    def to_dict(self):
        return {k: str(v) for k, v in self.model_dump().items()}


class TranscriptCompressionResult(BaseModel):
    """Transcript compression result"""

    original: str
    compressed: str
    analysis: TranscriptAnalysis
    compression_ratio: float
    information_preserved: float
    metadata: dict


class TemporalPattern(BaseModel):
    """Represents extracted temporal information"""

    days: Optional[list[str]] = Field(default=None, description="days of the week")
    times: Optional[list[str]] = Field(default=None, description="times of day")
    duration: Optional[str] = Field(default=None, description="duration")
    frequency: Optional[str] = Field(default=None, description="frequency: 33x day")
    pattern: Optional[str] = Field(default=None, description="pattern")
    resolved_date: Optional[str] = Field(
        default=None, description="Resolved ISO date e.g. '2026-02-24'"
    )


class ThreadOutput(CLMOutput):
    @staticmethod
    def _parse_commitment(commitment_str: str) -> dict:
        _KNOWN_TYPES = [
            "CONFIRMATION_EMAIL",
            "TECHNICIAN_VISIT",
            "RESOLUTION",
            "DELIVERY",
            "CALLBACK",
            "FOLLOWUP",
            "MONITORING",
            "CREDIT",
            "REFUND",
        ]
        _TIMELINE_TO_DAYS = {
            "3-5_BUSINESS_DAYS": 4,
            "1-3_BUSINESS_DAYS": 2,
            "WITHIN_24_HOURS": 1,
            "WITHIN_48_HOURS": 2,
            "WITHIN_HOURS": 0,
            "TODAY": 0,
            "TOMORROW": 1,
        }
        commitment_type = commitment_str
        timeline_str = None
        for known_type in _KNOWN_TYPES:
            if commitment_str.startswith(known_type):
                commitment_type = known_type
                remainder = commitment_str[len(known_type) :]
                if remainder.startswith("_"):
                    timeline_str = remainder[1:]
                break
        result: dict = {"type": commitment_type}
        if timeline_str is not None:
            result["etaDays"] = _TIMELINE_TO_DAYS.get(timeline_str)
        return result

    def to_dict(self) -> dict:
        """Parse compressed tokens into a structured dictionary.

        All recognised token fields are present; absent tokens have value None.
        """
        result: dict = {
            "id": self.metadata.get("id"),
            "channel": None,
            "lang": None,
            "domain": None,
            "service": None,
            "customerIntent": None,
            "secondaryIntent": None,
            "state": None,
            "resolution": None,
            "sentiment": None,
            "agentActions": None,
            "systemActions": None,
            "commitments": None,
            "artifacts": None,
            "createdAt": self.metadata.get("created_at"),
            "durationSeconds": None,
            "supportTrigger": None,
            "context": None,
        }

        context_list: list[str] = []
        artifacts_list: list[dict] = []
        commitments_list: list[dict] = []

        for token in re.findall(r"\[([^\]]+)\]", self.compressed):
            if token.startswith("INTERACTION:"):
                parts = token.split(":")
                if len(parts) >= 3 and "=" in parts[2]:
                    result["channel"] = parts[2].split("=", 1)[1]
            elif token.startswith("DURATION="):
                duration_str = token[9:]
                if duration_str.endswith("m"):
                    try:
                        result["durationSeconds"] = int(duration_str[:-1]) * 60
                    except ValueError:
                        pass
            elif token.startswith("LANG="):
                result["lang"] = token[5:]
            elif token.startswith("DOMAIN:"):
                result["domain"] = token[7:]
            elif token.startswith("SERVICE:"):
                result["service"] = token[8:]
            elif token.startswith("CUSTOMER_INTENTS:"):
                for part in token[17:].split(";"):
                    if part.startswith("PRIMARY="):
                        result["customerIntent"] = part[8:]
                    elif part.startswith("SECONDARY="):
                        result["secondaryIntent"] = part[10:]
            elif token.startswith("CUSTOMER_INTENT:"):
                result["customerIntent"] = token[16:]
            elif token.startswith("INTERACTION_TRIGGER:"):
                result["interactionTrigger"] = token[16:]
            elif token.startswith("CONTEXT:"):
                context_list.append(token[8:])
            elif token.startswith("AGENT_ACTIONS:"):
                result["agentActions"] = token[14:].split("\u2192")
            elif token.startswith("SYSTEM_ACTIONS:"):
                result["systemActions"] = token[15:].split("\u2192")
            elif token.startswith("RESOLUTION:"):
                result["resolution"] = token[11:]
            elif token.startswith("STATE:"):
                result["state"] = token[6:]
            elif token.startswith("COMMITMENT:"):
                commitments_list.append(self._parse_commitment(token[11:]))
            elif token.startswith("ARTIFACT:"):
                artifact_str = token[9:]
                if "=" in artifact_str:
                    key, value = artifact_str.split("=", 1)
                    artifacts_list.append({"key": key, "value": value})
            elif token.startswith("SENTIMENT:"):
                result["sentiment"] = token[10:].split("\u2192")

        if context_list:
            result["context"] = context_list
        if artifacts_list:
            result["artifacts"] = artifacts_list
        if commitments_list:
            result["commitments"] = commitments_list

        return result



from ...utils.vocabulary import BaseVocabulary


class PTVocabulary(BaseVocabulary):
    @property
    def STOPWORDS(self) -> tuple[str, ...]:
        return (
            "ele",
            "ela",
            "isso",
            "isto",
            "aquilo",
            "esses",
            "essas",
            "um",
            "uma",
            "o",
            "a",
            "os",
            "as",
            "uns",
            "umas",
            "algum",
            "alguma",
            "alguns",
            "algumas",
            "todo",
            "toda",
            "todos",
            "todas",
            "nenhum",
            "nenhuma",
            "tal",
            "o seguinte",
            "algo",
            "qualquer coisa",
            "tudo",
            "nada",
            "alguém",
            "qualquer um",
            "ninguém",
        )

    @property
    def EPISTEMIC_KEYWORDS(self) -> dict[str, list[str]]:
        return {
            "future": [
                "próximo",
                "próxima",
                "futuro",
                "futura",
                "será",
                "esperado",
                "previsão",
                "predição",
                "vindouro",
            ],
            "uncertainty": [
                "chance",
                "probabilidade",
                "possibilidade",
                "risco",
                "talvez",
                "eventualidade",
            ],
            "real_world": [
                "jogo",
                "partida",
                "temporada",
                "clima",
                "mercado",
                "ação",
                "equipe",
                "time",
                "bolsa",
            ],
        }

    @property
    def CODE_INDICATORS(self) -> tuple[str, ...]:
        return (
            "código",
            "script",
            "função",
            "programa",
            "algoritmo",
            "trecho",
            "arquivo",
            "implementação",
            "sintaxe",
            "debugar",
            "depurar",
            "compilar",
            "tempo de execução",
            "biblioteca",
            "framework",
            "api",
            "classe",
            "método",
            "variável",
            "módulo",
            "pacote",
            "importar",
            "dependência",
            "namespace",
            "interface",
            "repositório",
            "repo",
            "git",
            "commit",
            "branch",
            "pull request",
            "pr",
            "merge",
            "fusão",
            "deploy",
            "build",
            "compilação",
            "teste unitário",
            "caso de teste",
            "cobertura",
            "lint",
            "refatorar",
        )

    @property
    def PRONOUNS(self) -> tuple[str, ...]:
        return (
            "eu",
            "nós",
            "tu",
            "você",
            "vocês",
            "eles",
            "elas",
            "ele",
            "ela",
            "me",
            "nos",
            "te",
            "lhe",
            "lhes",
            "o",
            "a",
            "os",
            "as",
            "meu",
            "minha",
            "meus",
            "minhas",
            "nosso",
            "nossa",
            "nossos",
            "nossas",
            "teu",
            "tua",
            "teus",
            "tuas",
            "seu",
            "sua",
            "seus",
            "suas",
            "dele",
            "dela",
            "deles",
            "delas",
        )

    @property
    def MODALS(self) -> tuple[str, ...]:
        return (
            "poder",
            "posso",
            "pode",
            "podemos",
            "podem",
            "poderia",
            "dever",
            "devo",
            "deve",
            "devemos",
            "devem",
            "deveria",
            "querer",
            "quero",
            "quer",
            "queremos",
            "querem",
            "gostaria",
            "ter que",
            "tenho que",
            "tem que",
            "precisa",
            "há de",
        )

    @property
    def ACTION_VERBS(self) -> tuple[str, ...]:
        return (
            "reduzir",
            "aumentar",
            "melhorar",
            "otimizar",
            "aprimorar",
            "atualizar",
            "modificar",
            "refatorar",
            "migrar",
            "corrigir",
            "solucionar",
            "resolver",
            "debugar",
            "depurar",
            "reparar",
            "diagnosticar",
            "patchear",
            "criar",
            "gerar",
            "construir",
            "fazer",
            "produzir",
            "projetar",
            "desenvolver",
            "implementar",
            "analisar",
            "examinar",
            "revisar",
            "avaliar",
            "validar",
            "verificar",
            "testar",
            "medir",
            "monitorar",
            "explicar",
            "descrever",
            "esclarecer",
            "definir",
            "documentar",
            "calcular",
            "computar",
            "determinar",
            "encontrar",
            "processar",
            "comparar",
            "contrastar",
            "diferenciar",
            "classificar",
            "categorizar",
            "ordenar",
            "organizar",
            "agrupar",
            "extrair",
            "transformar",
            "carregar",
            "agregar",
            "filtrar",
            "deployar",
            "liberar",
            "reverter",
            "escalar",
            "provisionar",
            "destacar",
            "identificar",
            "mostrar",
            "demonstrar",
            "exibir",
        )

    def __init__(self):
        super().__init__()

    @property
    def QUANTIFIER_WORDS(self) -> tuple[str, ...]:
        return ("todos", "todas", "tudo", "cada", "inteiro", "completo")

    @property
    def DEMONSTRATIVES(self) -> list[str]:
        return [
            "este",
            "esta",
            "isto",
            "esse",
            "essa",
            "isso",
            "aquele",
            "aquela",
            "aquilo",
        ]

    @property
    def COMPOUND_PHRASES(self) -> dict[str, str]:
        return {
            "suporte ao cliente": "TICKET",
            "ticket de suporte": "TICKET",
            "chamado de suporte": "TICKET",
            "mensagem de email": "EMAIL",
            "transcrição de chat": "TRANSCRIPT",
            "chamada telefônica": "CALL",
            "ligação telefônica": "CALL",
            "código fonte": "CODE",
            "plano de negócios": "PLAN",
            "descrição do produto": "DESCRIPTION",
        }

    @property
    def rank_triggers(self) -> set[str]:
        return {
            "classificar",
            "ranquear",
            "rankear",
            "ordenar",
            "ordem",
            "ordenar por",
            "classificar por",
            "priorizar",
            "topo",
            "superior",
            "primeiro",
            "inferior",
            "último",
            "fundo",
            "maior",
            "mais alto",
            "menor",
            "mais baixo",
            "melhor",
            "melhores",
            "pior",
            "piores",
        }

    @property
    def domain_candidates(self) -> dict[str, list[str]]:
        return {
            "CODE": ["bug", "erro", "segurança", "desempenho", "performance"],
            "ENTITIES": [
                "nomes",
                "datas",
                "valores",
                "quantias",
                "endereços",
                "emails",
                "telefones",
            ],
            "QA": [
                "verificação",
                "política",
                "habilidades interpessoais",
                "soft skills",
                "precisão",
                "exatidão",
                "conformidade",
                "compliance",
                "divulgações",
            ],
            "SUPPORT": [
                "problema",
                "questão",
                "sentimento",
                "ações",
                "urgência",
                "prioridade",
                "ticket",
                "caso",
                "cliente",
                "suporte",
                "chamada",
                "reclamação",
                "atendente",
                "agente",
                "chamador",
                "chat",
                "consulta",
                "solução de problemas",
            ],
            "TECHNICAL": [
                "erro",
                "bug",
                "stacktrace",
                "exceção",
                "api",
                "servidor",
                "log",
                "depurar",
                "rastreamento",
                "falha",
                "implantação",
                "backend",
            ],
            "DOCUMENT": [
                "documento",
                "artigo",
                "manual",
                "guia",
                "doc",
                "texto",
                "transcrição",
                "notas",
                "resumo",
                "instruções",
            ],
            "BUSINESS": [
                "relatório",
                "briefing",
                "análise",
                "executivo",
                "gestão",
                "dashboard",
                "kpi",
                "roi",
                "funil",
                "trimestral",
                "apresentação",
            ],
            "LEGAL": [
                "contrato",
                "política",
                "conformidade",
                "gdpr",
                "cláusula",
                "advogado",
                "acordo",
                "termos",
                "privacidade",
            ],
            "FINANCE": [
                "fatura",
                "cobrança",
                "pagamento",
                "transação",
                "reembolso",
                "despesa",
                "saldo",
                "extrato",
                "valor",
            ],
            "SECURITY": [
                "violação",
                "risco",
                "ameaça",
                "alerta",
                "malware",
                "phishing",
                "permissões",
                "controle de acesso",
                "auditoria",
            ],
            "MEDICAL": [
                "paciente",
                "diagnóstico",
                "prescrição",
                "clínico",
                "prontuário",
                "sintomas",
                "tratamento",
                "médico",
            ],
            "SALES": [
                "lead",
                "crm",
                "oportunidade",
                "pipeline",
                "gerente de contas",
                "prospecto",
                "negócio",
                "cotação",
            ],
            "EDUCATION": [
                "lição",
                "currículo",
                "professor",
                "aluno",
                "treinamento",
                "curso",
                "aprendizagem",
            ],
        }

    @property
    def CONCEPT_INDICATORS(self) -> list[str]:
        """Optional - can have sensible default"""
        return ["conceito de", "ideia de", "noção de", "princípio de"]

    @property
    def MEETING_WORDS(self) -> tuple[str, ...]:
        """Optional - can have empty default"""
        return ("reunião", "conferência", "encontro", "sessão")

    @property
    def PROPOSAL_WORDS(self) -> list[str]:
        """Optional - can have empty default"""
        return ["proposta", "proposição"]

    @property
    def ARTICLES(self) -> list[str]:
        """Optional - can have empty default"""
        return ["o", "a", "os", "as", "um", "uma", "uns", "umas"]

    @property
    def REQ_TOKENS(self) -> dict[str, list[str]]:
        return {
            "ANALYZE": [
                "analisar",
                "analise",
                "revisar",
                "examinar",
                "avaliar",
                "inspecionar",
                "verificar",
                "auditar",
                "investigar",
                "conferir",
            ],
            "MATCH": [
                "combinar",
                "comparar",
                "alinhar",
                "mapear",
                "correlacionar",
                "corresponder",
                "cruzar",
                "casar",
                "verificar contra",
            ],
            "EXTRACT": [
                "extrair",
                "retirar",
                "identificar",
                "encontrar",
                "localizar",
                "obter",
                "recuperar",
                "retornar",
                "incluir",
                "selecionar",
                "pegar",
                "buscar",
            ],
            "SELECT": [
                "selecionar",
                "escolher",
                "filtrar",
                "identificar correspondentes",
                "optar",
                "eleger",
            ],
            "GENERATE": [
                "gerar",
                "criar",
                "escrever",
                "redigir",
                "compor",
                "produzir",
                "construir",
                "desenvolver",
                "projetar",
                "elaborar",
                "fazer",
                "nomear",
                "sugerir",
                "formular",
                "formar",
                "propor",
                "transcrever",
                "montar",
            ],
            "SUMMARIZE": [
                "resumir",
                "condensar",
                "sintetizar",
                "sumarizar",
                "recapitular",
                "abreviar",
                "compendiar",
            ],
            "TRANSFORM": [
                "converter",
                "transformar",
                "mudar",
                "reescrever",
                "traduzir",
                "completar",
                "modificar",
                "adaptar",
                "ajustar",
                "reformular",
                "alterar",
                "editar",
                "adicionar",
                "parafrasear",
                "preencher",
                "remover",
                "substituir",
                "inverter",
                "trocar",
            ],
            "EXPLAIN": [
                "explicar",
                "descrever",
                "esclarecer",
                "elaborar",
                "detalhar",
                "expor",
                "ilustrar",
                "expressar",
                "contar",
                "discutir",
                "definir",
                "elucidar",
                "falar sobre",
                "me diga sobre",
            ],
            "COMPARE": [
                "comparar",
                "contrastar",
                "versus",
                "vs",
                "diferença entre",
                "diferenciar",
                "distinguir",
                "confrontar",
            ],
            "CLASSIFY": [
                "classificar",
                "categorizar",
                "ordenar",
                "agrupar",
                "rotular",
                "organizar",
                "arranjar",
                "segmentar",
                "separar por",
            ],
            "DEBUG": [
                "debugar",
                "depurar",
                "diagnosticar",
                "corrigir bug",
                "investigar bug",
                "encontrar bug",
                "rastrear",
                "identificar problema",
                "resolver erro",
            ],
            "OPTIMIZE": [
                "otimizar",
                "melhorar",
                "aprimorar",
                "refatorar",
                "acelerar",
                "simplificar",
                "maximizar",
                "minimizar",
                "reduzir",
                "aumentar",
                "aperfeiçoar",
            ],
            "VALIDATE": [
                "validar",
                "verificar",
                "checar",
                "confirmar",
                "testar",
                "garantir",
                "certificar",
                "autenticar",
                "assegurar",
            ],
            "SEARCH": [
                "buscar",
                "pesquisar",
                "procurar",
                "encontrar",
                "localizar",
                "consultar",
                "descobrir",
                "achar",
            ],
            "RANK": [
                "priorizar",
                "ordenar",
                "classificar por",
                "pontuar",
                "ranquear",
                "avaliar",
                "hierarquizar",
            ],
            "PREDICT": [
                "prever",
                "projetar",
                "estimar",
                "antecipar",
                "prognosticar",
                "extrapolar",
                "predizer",
                "vaticinar",
            ],
            "FORMAT": [
                "formatar",
                "estruturar",
                "organizar",
                "dispor",
                "arranjar",
                "layoutar",
            ],
            "DETECT": [
                "detectar",
                "identificar",
                "descobrir",
                "perceber",
                "reconhecer",
                "notar",
                "captar",
            ],
            "CALCULATE": [
                "calcular",
                "computar",
                "determinar",
                "quantificar",
                "medir",
                "contar",
                "somar",
                "totalizar",
            ],
            # Aggregation
            "AGGREGATE": [
                "agregar",
                "agrupar",
                "combinar",
                "consolidar",
                "juntar",
                "mesclar",
                "compilar",
                "coletar",
                "reunir",
            ],
            "DETERMINE": [
                "determinar",
                "decidir",
                "avaliar",
                "concluir",
                "estabelecer",
                "escolher",
                "definir",
                "resolver",
            ],
            "ROUTE": [
                "rotear",
                "atribuir",
                "direcionar",
                "encaminhar",
                "enviar para",
                "delegar",
                "alocar",
                "distribuir",
            ],
            "EXECUTE": [
                "usar",
                "aplicar",
                "implementar",
                "executar",
                "realizar",
                "empregar",
                "utilizar",
                "rodar",
            ],
            "LIST": [
                "listar",
                "enumerar",
                "itemizar",
                "delinear",
                "relacionar",
                "elencar",
            ],
        }

    @property
    def NOISE_VERBS(self) -> set[str]:
        return {
            "ser",
            "estar",
            "ter",
            "haver",
            "poder",
            "dever",
            "querer",
            "ir",
            "vir",
            "fazer",
            "ficar",
            "viver",
            "morar",
            "continuar",
            "permanecer",
            "seguir",
            "basear",
            "precisar",
            "necessitar",
            "começar",
            "iniciar",
            "tornar",
            "parecer",
            "aparecer",
            "mostrar",
            "ver",
            "saber",
            "pensar",
            "sentir",
            "dizer",
            "falar",
            "chamar",
        }

    @property
    def CONTEXT_FILTERS(self) -> dict[str, list[str]]:
        return {
            "dar": ["dado", "dando", "dê-me", "me dê"],
            "seguir": ["seguinte", "a seguir", "seguindo"],
            "basear": ["baseado em", "com base em"],
            "usar": ["útil", "usado para", "utilizado"],
        }

    @property
    def TARGET_TOKENS(self) -> dict[str, list[str]]:
        return {
            "CODE": [
                "código",
                "script",
                "programa",
                "função",
                "método",
                "classe",
                "trecho",
                "rotina",
            ],
            "DATA": [
                "dados",
                "dataset",
                "conjunto de dados",
                "banco de dados",
                "planilha",
                "tabela",
                "csv",
            ],
            "QUERY": [
                "query",
                "consulta",
                "sql",
                "consulta sql",
                "consulta de banco",
            ],
            "ENDPOINT": [
                "endpoint",
                "api",
                "ponto de acesso",
                "serviço rest",
            ],
            "COMPONENT": [
                "componente",
                "módulo",
                "pacote",
                "biblioteca",
                "lib",
            ],
            "SYSTEM": [
                "sistema",
                "aplicação",
                "app",
                "aplicativo",
                "software",
                "plataforma",
            ],
            "TEST": [
                "teste",
                "teste unitário",
                "caso de teste",
                "suíte de testes",
            ],
            "LOG": [
                "log",
                "logs",
                "arquivo de log",
                "registro",
                "histórico",
            ],
            "RECORD": [
                "registro",
                "entrada",
                "linha",
                "item",
            ],
            "STRATEGY": [
                "estratégia",
                "abordagem",
                "plano",
                "metodologia",
            ],
            "NBA_CATALOG": [
                "nba",
                "nbas",
                "próxima melhor ação",
                "próximas melhores ações",
                "ações predefinidas",
                "ações possíveis",
                "ações disponíveis",
            ],
            "DOCUMENT": [
                "documento",
                "doc",
                "arquivo",
                "relatório",
                "artigo",
                "papel",
            ],
            "EMAIL": [
                "email",
                "e-mail",
                "mensagem",
                "correspondência",
            ],
            "REPORT": [
                "relatório",
                "análise",
                "conclusões",
                "parecer",
            ],
            "TICKET": [
                "ticket",
                "chamado",
                "ocorrência",
                "caso",
                "solicitação",
            ],
            "TRANSCRIPT": [
                "transcrição",
                "conversa",
                "diálogo",
                "histórico de chat",
                "atendimento",
            ],
            "FEEDBACK": [
                "feedback",
                "comentário",
                "avaliação",
                "crítica",
                "retorno",
            ],
            "COMMENT": [
                "comentário",
                "observação",
                "nota",
                "anotação",
            ],
            "CUSTOMER_INTENT": [
                "intenção do cliente",
                "necessidade do cliente",
                "pedido do cliente",
                "objetivo do cliente",
                "problema do cliente",
            ],
            "COMPLAINT": [
                "reclamação",
                "queixa",
                "problema",
                "insatisfação",
            ],
            "REQUEST": [
                "solicitação",
                "pedido",
                "requisição",
            ],
            "INQUIRY": [
                "consulta",
                "pergunta",
                "dúvida",
                "questionamento",
            ],
            "INTERACTION": [
                "interação",
                "atendimento",
                "contato",
            ],
            "CALL": [
                "chamada",
                "ligação",
                "telefonema",
            ],
            "DESCRIPTION": [
                "descrição",
                "detalhamento",
            ],
            "SUMMARY": [
                "resumo",
                "sumário",
                "síntese",
            ],
            "PLAN": [
                "plano",
                "planejamento",
                "projeto",
            ],
            "POST": [
                "post",
                "postagem",
                "publicação",
            ],
            "PRESS_RELEASE": [
                "comunicado de imprensa",
                "release",
                "nota à imprensa",
                "anúncio",
            ],
            "INTRODUCTION": [
                "introdução",
                "apresentação",
            ],
            "CAPTION": [
                "legenda",
                "caption",
            ],
            "QUESTIONS": [
                "perguntas",
                "questões",
                "questionário",
                "faq",
            ],
            "RESPONSE": [
                "resposta",
                "réplica",
                "retorno",
                "template de resposta",
            ],
            "LOGS": [
                "logs",
                "registros",
                "históricos",
            ],
            "CORRELATION": [
                "correlação",
                "correlações",
                "relação",
                "relações",
            ],
            "TRADEOFF": [
                "trade-off",
                "compensação",
                "custo-benefício",
            ],
            "PARADIGM": [
                "paradigma",
                "paradigmas",
                "modelo",
            ],
            "PAIN_POINTS": [
                "pontos de dor",
                "dores",
                "problemas",
                "dificuldades",
            ],
            "PATTERN": [
                "padrão",
                "padrões",
                "tendência",
                "tendências",
            ],
            "CHURN": [
                "churn",
                "cancelamento",
                "evasão",
                "rotatividade",
            ],
            "FEATURES": [
                "funcionalidades",
                "recursos",
                "características",
                "features",
            ],
            "METRICS": [
                "receita",
                "métricas",
                "estatísticas",
                "números",
                "indicadores",
            ],
            "REGIONS": [
                "regiões",
                "áreas",
                "locais",
                "territórios",
            ],
            "ITEMS": [
                "itens",
                "coisas",
                "elementos",
                "lista",
                "opções",
                "escolhas",
            ],
            "CONCEPT": [
                "conceito",
                "ideia",
                "noção",
                "princípio",
                "teoria",
            ],
            "PROCEDURE": [
                "procedimento",
                "processo",
                "passos",
                "método",
                "técnica",
            ],
            "FACT": [
                "fato",
                "fatos",
                "informação",
                "informações",
                "detalhes",
            ],
            "ANSWER": [
                "resposta",
                "solução",
            ],
        }

    @property
    def EXTRACT_FIELDS(self) -> tuple[str, ...]:
        return (
            "ISSUE",
            "SENTIMENT",
            "ACTIONS",
            "NEXT_STEPS",
            "URGENCY",
            "PRIORITY",
            "NAMES",
            "DATES",
            "AMOUNTS",
            "EMAILS",
            "PHONES",
            "ADDRESSES",
            "BUGS",
            "SECURITY",
            "PERFORMANCE",
            "STYLE",
            "ERRORS",
            "WARNINGS",
            "KEYWORDS",
            "TOPICS",
            "ENTITIES",
            "FACTS",
            "DECISIONS",
            "DEADLINES",
            "REQUIREMENTS",
            "FEATURES",
            "PROBLEMS",
            "SOLUTIONS",
            "RISKS",
            "METRICS",
            "KPI",
            "SCORES",
            "RATINGS",
            "FEEDBACK",
            "COMPLAINTS",
            "OWNERS",
            "ASSIGNEES",
            "STAKEHOLDERS",
            "PARTICIPANTS",
            "TIMESTAMPS",
            "DURATIONS",
            "FREQUENCIES",
            "QUANTITIES",
            "CATEGORIES",
            "TAGS",
            "LABELS",
            "STATUS",
            "TYPE",
            "CUSTOMER_INTENT",
            "RELEVANCE_SCORE",
            "NBA_ID",
            "MATCH_CONFIDENCE",
            "SEMANTIC_SIMILARITY",
            "THRESHOLD",
        )

    @property
    def OUTPUT_FORMATS(self) -> dict[str, list[str]]:
        return {
            "JSON": ["json", "formato json"],
            "MARKDOWN": ["markdown", "md"],
            "TABLE": ["tabela", "tabular"],
            "LIST": ["lista", "tópicos", "bullet points"],
            "PLAIN": ["texto simples", "texto puro"],
            "CSV": ["csv", "separado por vírgula"],
        }

    @property
    def IMPERATIVE_PATTERNS(self) -> list[tuple[list[str], str, str]]:
        return [
            (["liste", "listar", "enumere", "enumerar", "itemize"], "LIST", "ITEMS"),
            (["nomeie", "nomear", "identifique", "identificar"], "GENERATE", "ITEMS"),
            (
                ["dê", "dar", "forneça", "fornecer", "sugira", "sugerir"],
                "GENERATE",
                "ITEMS",
            ),
            (
                [
                    "diga",
                    "dizer",
                    "explique",
                    "explicar",
                    "descreva",
                    "descrever",
                    "esclareça",
                    "esclarecer",
                    "ilustre",
                    "ilustrar",
                ],
                "EXPLAIN",
                "CONCEPT",
            ),
        ]

    @property
    def QUESTION_WORDS(self) -> list[str]:
        return [
            "o que",
            "que",
            "quem",
            "onde",
            "quando",
            "por que",
            "porque",
            "como",
            "qual",
            "quais",
        ]



from clm_core.utils.parser_rules import BaseRules


class ESRules(BaseRules):
    """Spanish language pattern matching rules"""

    def __init__(self):
        super().__init__()

    @property
    def COMPARISON_MAP(self) -> dict[str, str]:
        return {
            r"\bdiferencias?\b": "DIFFERENCES",
            r"\bdistinguir\b": "DIFFERENCES",
            r"\bcontrastar\b": "DIFFERENCES",
            r"\bsimilitudes?\b": "SIMILARITIES",
            r"\bsemejanzas?\b": "SIMILARITIES",
            r"\bcomún\b": "SIMILARITIES",
            r"\bpros\s*(y|&)?\s*contras\b": "PROS_CONS",
            r"\bventajas\s*(y|&)?\s*desventajas\b": "PROS_CONS",
            r"\bbeneficios\s*(y|&)?\s*inconvenientes\b": "PROS_CONS",
            r"\bcompensaciones?\b": "TRADEOFFS",
            r"\btrade-?offs?\b": "TRADEOFFS",
        }

    @property
    def DOMAIN_REGEX(self) -> dict[str, str]:
        return {
            "SUPPORT": r"\b(llamada|ticket|caso|cliente|soporte|queja|helpdesk)\b",
            "TECHNICAL": r"\b(error|bug|fallo|stacktrace|api|depuración|servidor|excepción)\b",
            "DOCUMENT": r"\b(documento|artículo|manual|transcripción|guía)\b",
            "BUSINESS": r"\b(informe|ejecutivo|análisis|kpi|panel)\b",
            "LEGAL": r"\b(contrato|política|cumplimiento|cláusula|rgpd)\b",
            "FINANCE": r"\b(factura|cobro|pago|reembolso|transacción)\b",
            "SECURITY": r"\b(brecha|amenaza|riesgo|malware|auditoría)\b",
            "MEDICAL": r"\b(paciente|clínico|diagnóstico|tratamiento)\b",
            "SALES": r"\b(lead|crm|oportunidad|prospecto)\b",
            "EDUCATION": r"\b(lección|profesor|estudiante|currículo)\b",
        }

    @property
    def DURATION_PATTERNS(self) -> list[str]:
        return [
            r"(\d+)[\s-]*(?:minuto|min)s?",
            r"(\d+)[\s-]*(?:hora|hr)s?",
        ]

    @property
    def STANDARD_FIELD_KEYWORDS(self) -> dict[str, str]:
        return {
            r"\bproblema(s)?\b": "ISSUE",
            r"\bcuestión(es)?\b": "PROBLEM",
            r"\berror(es)?\b": "ERROR",
            r"\bbug(s)?\b": "BUG",
            r"\bnombre(s)?\b": "NAMES",
            r"\bfecha(s)?\b": "DATES",
            r"\bcantidad(es)?\b": "AMOUNTS",
            r"\bmonto(s)?\b": "AMOUNTS",
            r"\bemail(s)?\b": "EMAILS",
            r"\bcorreo(s)?\b": "EMAILS",
            r"\bteléfono(s)?\b": "PHONES",
            r"\bdirección(es)?\b": "ADDRESSES",
            r"\bsentimiento\b": "SENTIMENT",
            r"\burgencia\b": "URGENCY",
            r"\bprioridad\b": "PRIORITY",
            r"\bcategoría\b": "CATEGORY",
            r"\bacción(es)?\b": "ACTIONS",
            r"\bpróximos pasos\b": "NEXT_STEPS",
            r"\bsiguientes pasos\b": "NEXT_STEPS",
            r"\belementos de acción\b": "ACTIONS",
            r"\bfecha(s)? límite\b": "DEADLINES",
            r"\bplazo(s)?\b": "DEADLINES",
            r"\bseguridad\b": "SECURITY",
            r"\brendimiento\b": "PERFORMANCE",
            r"\bdesempeño\b": "PERFORMANCE",
        }

    @property
    def AUDIENCE_MAP(self) -> dict[str, str]:
        return {
            r"\bno[- ]?técnico\b": "NON_TECHNICAL",
            r"\b(principiante|principiantes|niño|niños|simple|general)\b": "BEGINNER",
            r"\b(experto|avanzado|especialista|profesionales)\b": "EXPERT",
            r"\btécnico\b": "TECHNICAL",
            r"\b(negocio|ejecutivos|gerencia|dirección)\b": "BUSINESS",
        }

    @property
    def LENGTH_MAP(self) -> dict[str, str]:
        return {
            r"\b(breve|corto|conciso|rápido|resumen)\b": "BRIEF",
            r"\b(detallado|completo|exhaustivo|profundo|extenso)\b": "DETAILED",
        }

    @property
    def STYLE_MAP(self) -> dict[str, str]:
        return {
            r"\b(simple|fácil|sencillo)\b": "SIMPLE",
            r"\b(formal|empresarial|profesional)\b": "FORMAL",
        }

    @property
    def TONE_MAP(self) -> dict[str, str]:
        return {
            r"\bprofesional\b": "PROFESSIONAL",
            r"\bformal\b": "PROFESSIONAL",
            r"\bempresarial\b": "PROFESSIONAL",
            r"\bcasual\b": "CASUAL",
            r"\binformal\b": "CASUAL",
            r"\bamigable\b": "CASUAL",
            r"\bempático\b": "EMPATHETIC",
            r"\bcompasivo\b": "EMPATHETIC",
            r"\bcomprensivo\b": "EMPATHETIC",
        }

    @property
    def NUMBER_WORDS(self) -> dict[str, int]:
        return {
            "uno": 1,
            "una": 1,
            "un": 1,
            "dos": 2,
            "par": 2,
            "tres": 3,
            "cuatro": 4,
            "cinco": 5,
            "seis": 6,
            "siete": 7,
            "ocho": 8,
            "nueve": 9,
            "diez": 10,
            "pocos": -1,
            "varios": -2,
            "muchos": -3,
        }

    @property
    def SPEC_PATTERNS(self) -> list[tuple[str, str]]:
        return [
            (r"(\d+)\s*líneas?\b", "LINES"),
            (r"(\d+)\s*palabras?\b", "WORDS"),
            (r"(\d+)\s*(?:elementos?|cosas?|ítems?)\b", "ITEMS"),
            (r"(\d+)\s*(?:consejos?|sugerencias?)\b", "COUNT"),
            (r"(\d+)\s*(?:ejemplos?|instancias?)\b", "COUNT"),
            (r"(\d+)\s*(?:pasos?|etapas?)\b", "STEPS"),
            (r"(\d+)\s*(?:formas?|métodos?|maneras?)\b", "COUNT"),
        ]

    @property
    def PROGRAMMING_LANGUAGE_PATTERN(self) -> list[tuple[str, str]]:
        return [
            (r"\bpython\b", "PYTHON"),
            (r"\bdjango\b", "PYTHON"),
            (r"\bflask\b", "PYTHON"),
            (r"\bpandas\b", "PYTHON"),
            (r"\b\.py\b", "PYTHON"),
            (r"\bjavascript\b", "JAVASCRIPT"),
            (r"\bnode\.?js\b", "JAVASCRIPT"),
            (r"\breact\b", "JAVASCRIPT"),
            (r"\bvue\b", "JAVASCRIPT"),
            (r"\bangular\b", "JAVASCRIPT"),
            (r"\b\.js\b", "JAVASCRIPT"),
            (r"\bjava\b(?!script)", "JAVA"),
            (r"\bspring\b", "JAVA"),
            (r"\bmaven\b", "JAVA"),
            (r"\bc\+\+\b", "CPP"),
            (r"\bcpp\b", "CPP"),
            (r"\bgolang\b", "GO"),
            (r"\bgo\s+(?:código|program|script)", "GO"),
            (r"\brust\b", "RUST"),
            (r"\btypescript\b", "TYPESCRIPT"),
            (r"\b\.ts\b", "TYPESCRIPT"),
        ]

    @property
    def EXTRACTION_INDICATORS(self) -> list[str]:
        return [
            r"\bextraer\b",
            r"\bidentificar\b",
            r"\bencontrar\b",
            r"\bobtener\b",
            r"\bsacar\b",
            r"\bresaltar\b",
            r"\bmostrar\b",
            r"\bdevolver\b",
            r"\brecuperar\b",
            r"\bcomparar\b",
            r"\bcontrastar\b",
            r"\blistar\b",
            r"\bcuáles son\b",
            r"\bqué son\b",
        ]

    @property
    def QA_CRITERIA(self) -> dict[str, str]:
        return {
            r"\b(verificación|verificar|verificado)\b": "VERIFICATION",
            r"\b(política|políticas|adherencia a política)\b": "POLICY",
            r"\b(habilidades blandas|empatía|claridad|responsabilidad)\b": "SOFT_SKILLS",
            r"\b(precisión|preciso|exactitud)\b": "ACCURACY",
            r"\b(cumplimiento|cumple|violaciones?)\b": "COMPLIANCE",
            r"\b(sentimiento|emoción|estado de ánimo)\b": "SENTIMENT",
            r"\b(divulgaciones?|divulgaciones obligatorias?)\b": "DISCLOSURES",
        }

    @property
    def QA_INDICATORS(self) -> list[str]:
        return [
            r"\bpuntuación\b",
            r"\bpuntaje\b",
            r"\bqa\b",
            r"\baseguramiento de calidad\b",
            r"\bcumplimiento\b",
            r"\bauditoría\b",
        ]

    @property
    def QUESTION_PATTERNS(self) -> list[tuple[str, int]]:
        return [
            (
                r"qué (?:es|son|significa) (?:el |la |los |las |un |una )?([\w\s]+?)(?:\?|$)",
                1,
            ),
            (r"cómo (?:funciona|funcionan|puede|pueden) ([\w\s]+?)(?:\?|$)", 1),
            (r"por qué (?:es|son|está|están) ([\w\s]+?)(?:\?|$)", 1),
            (r"dónde (?:está|están|se encuentra) ([\w\s]+?)(?:\?|$)", 1),
            (r"cuándo (?:es|fue|será) ([\w\s]+?)(?:\?|$)", 1),
            (r"quién (?:es|son|fue|fueron) (?:el |la )?([\w\s]+?)(?:\?|$)", 1),
        ]

    @property
    def EXPLAIN_PATTERNS(self) -> list[tuple[str, int]]:
        return [
            (r"(?:explica|describe) cómo ([\w\s]+?)(?: funciona| trabaja|$)", 1),
            (
                r"(?:explica|describe|aclara|detalla) (?:el |la |los |las |un |una )?([\w\s]+?)(?:\s+en|\s+con|\s+para|\.|\?|$)",
                1,
            ),
            (r"(?:cuéntame|dime) sobre ([\w\s]+?)(?:\s+en|\s+con|\.|\?|$)", 1),
        ]

    @property
    def CONCEPT_PATTERN(self) -> tuple[str, int]:
        return (
            r"(?:concepto|idea|noción|principio|teoría) de ([\w\s]+?)(?:\s+en|\.|\?|$)",
            1,
        )

    @property
    def PROCEDURE_PATTERN(self) -> tuple[str, int]:
        return (
            r"cómo (?:puedo|se puede|hago para) ([\w\s]+?)(?:\s+en|\s+con|\.|\?|$)",
            1,
        )

    @property
    def CLEANUP_TAIL(self) -> str:
        return r"\s+(de|en|para|con|sobre|desde|a|al|por|detalle|detalles|técnico|específico)$"

    @property
    def SUBJECT_PATTERNS(self) -> list[tuple[str, str]]:
        return [
            (r"\bverbo[s]?\b", "VERB"),
            (r"\bsustantivo[s]?\b", "NOUN"),
            (r"\badjetivo[s]?\b", "ADJECTIVE"),
            (r"\badverbio[s]?\b", "ADVERB"),
            (r"\bpronombre[s]?\b", "PRONOUN"),
            (r"\bpreposición(es)?\b", "PREPOSITION"),
            (r"\bconjunción(es)?\b", "CONJUNCTION"),
            (r"\bconsejo[s]?\b", "TIP"),
            (r"\bsugerencia[s]?\b", "TIP"),
            (r"\bmétodo[s]?\b", "METHOD"),
            (r"\btécnica[s]?\b", "TECHNIQUE"),
            (r"\bestrategia[s]?\b", "STRATEGY"),
            (r"\benfoque[s]?\b", "APPROACH"),
            (r"\bpráctica[s]?\b", "PRACTICE"),
            (r"\balgoritmo[s]?\b", "ALGORITHM"),
            (r"\bfunción(es)?\b", "FUNCTION"),
            (r"\bfórmula[s]?\b", "FORMULA"),
            (r"\becuación(es)?\b", "EQUATION"),
            (r"\bteorema[s]?\b", "THEOREM"),
            (r"\bprueba[s]?\b", "PROOF"),
            (r"\bejemplo[s]?\b", "EXAMPLE"),
            (r"\bidea[s]?\b", "IDEA"),
            (r"\bforma[s]?\b", "METHOD"),
            (r"\bmanera[s]?\b", "METHOD"),
            (r"\bpaso[s]?\b", "STEP"),
            (r"\bfactor(es)?\b", "FACTOR"),
            (r"\brazón(es)?\b", "REASON"),
            (r"\bmotivo[s]?\b", "REASON"),
            (r"\bbeneficio[s]?\b", "BENEFIT"),
            (r"\bventaja[s]?\b", "ADVANTAGE"),
            (r"\bdesventaja[s]?\b", "DISADVANTAGE"),
            (r"\bcaracterística[s]?\b", "FEATURE"),
            (r"\bmétrica[s]?\b", "METRIC"),
            (r"\bindicador(es)?\b", "INDICATOR"),
            (r"\bpercepción(es)?\b", "INSIGHT"),
            (r"\bhallazgo[s]?\b", "FINDING"),
        ]

    @property
    def TYPE_MAP(self) -> dict[str, str]:
        return {
            "llamada": "CALL",
            "llamada telefónica": "CALL",
            "reunión": "MEETING",
            "junta": "MEETING",
            "chat": "CHAT",
            "conversación": "CONVERSATION",
            "informe": "REPORT",
            "reporte": "REPORT",
            "artículo": "ARTICLE",
        }

    @property
    def CONTEXT_MAP(self) -> dict[str, str]:
        return {
            "cliente": "CUSTOMER",
            "soporte": "SUPPORT",
            "ventas": "SALES",
            "técnico": "TECHNICAL",
        }

    @property
    def ISSUE_PATTERNS(self) -> list[str]:
        return [
            r"sobre\s+([\w\s]+?)(?:\s+y|$)",
            r"respecto a\s+([\w\s]+?)(?:\s+y|$)",
            r"acerca de\s+([\w\s]+?)(?:\s+y|$)",
            r"relacionado con\s+([\w\s]+?)(?:\s+y|$)",
        ]



import re
from abc import ABC, abstractmethod


class BaseRules(ABC):
    """
    Abstract base class for language-specific pattern matching rules.
    Contains shared compilation logic while language classes define the patterns.
    """

    def __init__(self):
        """Initialize and compile all patterns on instantiation"""
        self._compile_patterns()

    @property
    @abstractmethod
    def DOMAIN_REGEX(self) -> dict[str, str]:
        raise NotImplementedError("Subclasses must implement DOMAIN_REGEX")

    @property
    @abstractmethod
    def COMPARISON_MAP(self) -> dict[str, str]:
        """Comparison keywords: differences, similarities, pros/cons, tradeoffs"""
        pass

    @property
    @abstractmethod
    def DURATION_PATTERNS(self) -> list[str]:
        """Regex patterns for duration extraction (minutes, hours)"""
        pass

    @property
    @abstractmethod
    def STANDARD_FIELD_KEYWORDS(self) -> dict[str, str]:
        """Standard extraction fields: issue, problem, error, names, dates, etc."""
        pass

    @property
    @abstractmethod
    def AUDIENCE_MAP(self) -> dict[str, str]:
        """Target audience keywords: non-technical, beginner, expert, etc."""
        pass

    @property
    @abstractmethod
    def LENGTH_MAP(self) -> dict[str, str]:
        """Length preferences: brief, short, detailed, comprehensive"""
        pass

    @property
    @abstractmethod
    def STYLE_MAP(self) -> dict[str, str]:
        """Style preferences: simple, formal, professional"""
        pass

    @property
    @abstractmethod
    def TONE_MAP(self) -> dict[str, str]:
        """Tone preferences: professional, casual, empathetic"""
        pass

    @property
    @abstractmethod
    def NUMBER_WORDS(self) -> dict[str, int]:
        """Number words mapping: one->1, two->2, few->-1, etc."""
        pass

    @property
    @abstractmethod
    def SPEC_PATTERNS(self) -> list[tuple[str, str]]:
        """Specification patterns: (pattern, label) for lines, words, items, steps"""
        pass

    @property
    @abstractmethod
    def PROGRAMMING_LANGUAGE_PATTERN(self) -> list[tuple[str, str]]:
        """Programming language detection: (pattern, language)"""
        pass

    @property
    @abstractmethod
    def EXTRACTION_INDICATORS(self) -> list[str]:
        """Verbs indicating extraction intent: extract, identify, find, etc."""
        pass

    @property
    @abstractmethod
    def QA_CRITERIA(self) -> dict[str, str]:
        """QA/compliance criteria: verification, policy, compliance, etc."""
        pass

    @property
    @abstractmethod
    def QA_INDICATORS(self) -> list[str]:
        """QA indicators: score, qa, quality assurance, audit"""
        pass

    @property
    @abstractmethod
    def QUESTION_PATTERNS(self) -> list[tuple[str, int]]:
        """Question patterns: (regex, group_index)"""
        pass

    @property
    @abstractmethod
    def EXPLAIN_PATTERNS(self) -> list[tuple[str, int]]:
        """Explanation patterns: (regex, group_index)"""
        pass

    @property
    @abstractmethod
    def CONCEPT_PATTERN(self) -> tuple[str, int]:
        """Concept pattern: (regex, group_index)"""
        pass

    @property
    @abstractmethod
    def PROCEDURE_PATTERN(self) -> tuple[str, int]:
        """Procedure pattern: (regex, group_index)"""
        pass

    @property
    @abstractmethod
    def CLEANUP_TAIL(self) -> str:
        """Pattern for cleaning up trailing words"""
        pass

    @property
    @abstractmethod
    def SUBJECT_PATTERNS(self) -> list[tuple[str, str]]:
        """Subject type patterns: verb, noun, tip, method, etc."""
        pass

    @property
    @abstractmethod
    def TYPE_MAP(self) -> dict[str, str]:
        """Communication type mapping: call, meeting, chat, etc."""
        pass

    @property
    @abstractmethod
    def CONTEXT_MAP(self) -> dict[str, str]:
        """Context mapping: customer, support, sales, technical"""
        pass

    @property
    @abstractmethod
    def ISSUE_PATTERNS(self) -> list[str]:
        """Issue extraction patterns: about X, regarding X"""
        pass

    def _compile_patterns(self):
        """Compile all regex patterns for efficient matching"""
        self.COMPILED = {
            "comparison": [
                (re.compile(p, re.I), v) for p, v in self.COMPARISON_MAP.items()
            ],
            "standard": [
                (re.compile(p, re.I), v)
                for p, v in self.STANDARD_FIELD_KEYWORDS.items()
            ],
            "audience": [
                (re.compile(p, re.I), v) for p, v in self.AUDIENCE_MAP.items()
            ],
            "length": [(re.compile(p, re.I), v) for p, v in self.LENGTH_MAP.items()],
            "style": [(re.compile(p, re.I), v) for p, v in self.STYLE_MAP.items()],
            "tone": [(re.compile(p, re.I), v) for p, v in self.TONE_MAP.items()],
            "specs": [(re.compile(p, re.I), name) for p, name in self.SPEC_PATTERNS],
            "extraction_indicators": [
                re.compile(p, re.I) for p in self.EXTRACTION_INDICATORS
            ],
            "qa_criteria": [
                (re.compile(p, re.I), v) for p, v in self.QA_CRITERIA.items()
            ],
            "qa_indicators": [re.compile(p, re.I) for p in self.QA_INDICATORS],
            "questions": [
                (re.compile(p, re.I), group) for p, group in self.QUESTION_PATTERNS
            ],
            "explain": [
                (re.compile(p, re.I), group) for p, group in self.EXPLAIN_PATTERNS
            ],
            "concept": re.compile(self.CONCEPT_PATTERN[0], re.I),
            "procedure": re.compile(self.PROCEDURE_PATTERN[0], re.I),
            "subject_patterns": [
                (re.compile(p, re.I), label) for p, label in self.SUBJECT_PATTERNS
            ],
            "issue_patterns": [re.compile(p, re.I) for p in self.ISSUE_PATTERNS],
            "language_patterns": [
                (re.compile(p, re.I), lang)
                for p, lang in self.PROGRAMMING_LANGUAGE_PATTERN
            ],
        }

    @property
    def ctx_patterns(self) -> dict[str, list[tuple[str, str]]]:
        """
        Returns a dictionary of context patterns for different categories.
        Categories:
        - LANGUAGE
        - REGION
        - SLA
        - FORMAT
        - PRIORITY
        """
        return {
            "LANGUAGE": [
                (r"\benglish\b", "ENGLISH"),
                (r"\bspanish\b", "SPANISH"),
                (r"\bfrench\b", "FRENCH"),
                (r"\bgerman\b", "GERMAN"),
                (r"\bchinese\b", "CHINESE"),
                (r"\bjapanese\b", "JAPANESE"),
            ],
            "REGION": [
                (r"\b(us|usa|american)\b", "US"),
                (r"\b(uk|british|england)\b", "UK"),
                (r"\b(europe|eu)\b", "EU"),
                (r"\b(apac|asia pacific)\b", "APAC"),
                (r"\bcanada|ca\b", "CA"),
            ],
            "PRIORITY": [
                (r"\burgent\b", "URGENT"),
                (r"\bpriority\b", "HIGH_PRIORITY"),
                (r"\basap\b", "URGENT"),
                (r"\bimmediately\b", "URGENT"),
            ],
            "SLA": [
                (r"\bwithin\s+(\d+)\s*(hours|hrs|h)\b", "SLA_HOURS"),
                (r"\brespond by\b", "SLA_DEADLINE"),
            ],
            "FORMAT": [
                (r"\bbullet(s)?\b", "BULLET_POINTS"),
                (r"\bnumbered\b", "NUMBERED_LIST"),
                (r"\btable\b", "TABLE"),
                (r"\bparagraph\b", "PARAGRAPH"),
            ],
        }

    def match_comparison(self, text: str) -> str | None:
        """Match comparison type in text"""
        for pattern, value in self.COMPILED["comparison"]:
            if pattern.search(text):
                return value
        return None

    def match_standard_field(self, text: str) -> str | None:
        """Match standard extraction field in text"""
        for pattern, value in self.COMPILED["standard"]:
            if pattern.search(text):
                return value
        return None

    def match_audience(self, text: str) -> str | None:
        """Match target audience in text"""
        for pattern, value in self.COMPILED["audience"]:
            if pattern.search(text):
                return value
        return None

    def match_length(self, text: str) -> str | None:
        """Match length preference in text"""
        for pattern, value in self.COMPILED["length"]:
            if pattern.search(text):
                return value
        return None

    def match_style(self, text: str) -> str | None:
        """Match style preference in text"""
        for pattern, value in self.COMPILED["style"]:
            if pattern.search(text):
                return value
        return None

    def match_tone(self, text: str) -> str | None:
        """Match tone preference in text"""
        for pattern, value in self.COMPILED["tone"]:
            if pattern.search(text):
                return value
        return None

    def match_specs(self, text: str) -> list[tuple[str, int]]:
        """Match specification patterns (lines, words, items, etc.)"""
        results = []
        for pattern, name in self.COMPILED["specs"]:
            match = pattern.search(text)
            if match:
                results.append((name, int(match.group(1))))
        return results

    def match_programming_language(self, text: str) -> str | None:
        """Match programming language in text"""
        for pattern, lang in self.COMPILED["language_patterns"]:
            if pattern.search(text):
                return lang
        return None

    def has_extraction_indicator(self, text: str) -> bool:
        """Check if text has extraction indicators"""
        return any(
            pattern.search(text) for pattern in self.COMPILED["extraction_indicators"]
        )

    def match_qa_criteria(self, text: str) -> list[str]:
        """Match QA criteria in text"""
        results = []
        for pattern, value in self.COMPILED["qa_criteria"]:
            if pattern.search(text):
                results.append(value)
        return results

    def has_qa_indicator(self, text: str) -> bool:
        """Check if text has QA indicators"""
        return any(pattern.search(text) for pattern in self.COMPILED["qa_indicators"])

    def extract_question_subject(self, text: str) -> str | None:
        """Extract subject from question patterns"""
        for pattern, group in self.COMPILED["questions"]:
            match = pattern.search(text)
            if match:
                return match.group(group).strip()
        return None

    def extract_explain_subject(self, text: str) -> str | None:
        """Extract subject from explain patterns"""
        for pattern, group in self.COMPILED["explain"]:
            match = pattern.search(text)
            if match:
                return match.group(group).strip()
        return None

    def extract_concept(self, text: str) -> str | None:
        """Extract concept from text"""
        match = self.COMPILED["concept"].search(text)
        if match:
            return match.group(self.CONCEPT_PATTERN[1]).strip()
        return None

    def extract_procedure(self, text: str) -> str | None:
        """Extract procedure from text"""
        match = self.COMPILED["procedure"].search(text)
        if match:
            return match.group(self.PROCEDURE_PATTERN[1]).strip()
        return None

    def match_subject_pattern(self, text: str) -> str | None:
        """Match subject pattern label"""
        for pattern, label in self.COMPILED["subject_patterns"]:
            if pattern.search(text):
                return label
        return None

    def cleanup_tail(self, text: str) -> str:
        """Remove trailing prepositions and common words"""
        return re.sub(self.CLEANUP_TAIL, "", text, flags=re.I).strip()

    def extract_issue_context(self, text: str) -> str | None:
        """Extract issue context from patterns"""
        for pattern in self.COMPILED["issue_patterns"]:
            match = pattern.search(text)
            if match:
                return match.group(1).strip()
        return None

    def parse_number_word(self, word: str) -> int | None:
        """Parse number word to integer"""
        return self.NUMBER_WORDS.get(word.lower())



from ._minimizer import ConfigurationPromptMinimizer
from ._prompt_template_validator import BoundPromptValidator
from ._schemas import (
    Intent,
    Target,
    VOCAB_SIGNAL_MAP,
    REQ,
    Signal,
    Artifact,
    PromptTemplate,
    ValidationLevel,
    PromptMode,
)

__all__ = [
    "Intent",
    "Target",
    "REQ",
    "Artifact",
    "VOCAB_SIGNAL_MAP",
    "Signal",
    "PromptTemplate",
    "BoundPromptValidator",
    "ValidationLevel",
    "ConfigurationPromptMinimizer",
    "PromptMode",
]



import json
from datetime import datetime
from pathlib import Path

from clm_core.core.compressors.statistical.schemas import Pattern, PatternStats


class PatternDatabase:
    def __init__(self, db_path: str) -> None:
        self.db_path = Path(db_path)
        self.patterns: dict[str, Pattern] = {}
        self.load()

    def load(self) -> None:
        """Load patterns from disk"""
        if self.db_path.exists():
            with open(self.db_path, "r") as f:
                data = json.load(f)
                for pattern_dict in data.get("patterns", []):
                    if isinstance(pattern_dict.get("first_seen"), str):
                        pattern_dict["first_seen"] = datetime.fromisoformat(
                            pattern_dict["first_seen"]
                        )
                    if isinstance(pattern_dict.get("last_seen"), str):
                        pattern_dict["last_seen"] = datetime.fromisoformat(
                            pattern_dict["last_seen"]
                        )
                    pattern = Pattern(**pattern_dict)
                    self.patterns[pattern.id] = pattern

    def save(self) -> None:
        """Save patterns to disk"""
        stats = self.get_stats()
        data = {
            "patterns": [
                {
                    "id": p.id,
                    "pattern": p.pattern,
                    "frequency": p.frequency,
                    "first_seen": p.first_seen.isoformat(),
                    "last_seen": p.last_seen.isoformat(),
                    "compression_gain": p.compression_gain,
                    "domains": p.domains,
                    "examples": p.examples,
                    "version": p.version,
                }
                for p in self.patterns.values()
            ],
            "stats": {
                "total_patterns": stats.total_patterns,
                "total_uses": stats.total_uses,
                "total_tokens_saved": stats.total_tokens_saved,
                "avg_compression_gain": stats.avg_compression_gain,
                "most_used_pattern": {
                    "id": stats.most_used_pattern.id,
                    "pattern": stats.most_used_pattern.pattern,
                    "frequency": stats.most_used_pattern.frequency,
                    "first_seen": stats.most_used_pattern.first_seen.isoformat(),
                    "last_seen": stats.most_used_pattern.last_seen.isoformat(),
                    "compression_gain": stats.most_used_pattern.compression_gain,
                    "domains": stats.most_used_pattern.domains,
                    "examples": stats.most_used_pattern.examples,
                    "version": stats.most_used_pattern.version,
                }
                if stats.most_used_pattern
                else None,
            },
        }

        with open(self.db_path, "w") as f:
            json.dump(data, f, indent=2)

    def add_pattern(self, pattern: Pattern) -> None:
        """Add or update a pattern"""
        if pattern.id in self.patterns:
            # Update existing pattern
            existing = self.patterns[pattern.id]
            existing.frequency += pattern.frequency
            existing.last_seen = datetime.now()
        else:
            self.patterns[pattern.id] = pattern

    def add_patterns(self, patterns: list[Pattern]) -> None:
        """Bulk add patterns"""
        for pattern in patterns:
            self.add_pattern(pattern)
        self.save()

    def get_pattern(self, pattern_id: str) -> Pattern:
        return self.patterns[pattern_id]

    def get_top_patterns(
        self, n: int = 100, domain: str | None = None
    ) -> list[Pattern]:
        """Get top N patterns by value score"""
        patterns = list(self.patterns.values())

        # Filter by domain if specified
        if domain:
            patterns = [p for p in patterns if domain in p.domains]

        # Sort by value score
        patterns.sort(key=lambda p: p.value_score, reverse=True)

        return patterns[:n]

    def search_patterns(self, token_sequence: str) -> list[tuple[Pattern, int]]:
        """
        Find patterns that match the token sequence
        Returns list of (pattern, start_position) tuples
        """
        matches = []
        for pattern in self.patterns.values():
            pos = 0
            while pos < len(token_sequence):
                idx = token_sequence.find(pattern.pattern, pos)
                if idx == -1:
                    break
                matches.append((pattern, idx))
                pos = idx + len(pattern.pattern)

        matches.sort(key=lambda x: x[1])
        return matches

    def get_stats(self) -> PatternStats:
        """Get database statistics"""
        if not self.patterns:
            return PatternStats(
                total_patterns=0,
                total_uses=0,
                total_tokens_saved=0,
                avg_compression_gain=0.0,
                most_used_pattern=None,
            )

        total_uses = sum(p.frequency for p in self.patterns.values())
        total_tokens_saved = sum(
            p.frequency * p.compression_gain for p in self.patterns.values()
        )
        avg_compression_gain = sum(
            p.compression_gain for p in self.patterns.values()
        ) / len(self.patterns)
        most_used = max(self.patterns.values(), key=lambda p: p.frequency)

        return PatternStats(
            total_patterns=len(self.patterns),
            total_uses=total_uses,
            total_tokens_saved=int(total_tokens_saved),
            avg_compression_gain=avg_compression_gain,
            most_used_pattern=most_used,
        )



from clm_core.types import (
    CLMOutput,
    SDCompressionConfig,
    SysPromptConfig,
    CLMConfig,
    FieldImportance,
)
from clm_core.components.sys_prompt import PromptMode, PromptTemplate
from clm_core.components.ds_compression import SDEncoderV2, SDEncoder
from clm_core.components.sys_prompt.encoder import SysPromptEncoder
from clm_core.components.thread_encoder.encoder import ThreadEncoder
from clm_core.components.thread_encoder import (
    TranscriptCompressionResult,
    TranscriptAnalysis,
)
from clm_core.encoder import CLMEncoder

from .__version__ import __description__, __title__, __version__

__all__ = [
    "__version__",
    "__title__",
    "__description__",
    "CLMEncoder",
    "CLMConfig",
    "CLMOutput",
    "SysPromptConfig",
    "SDCompressionConfig",
    "FieldImportance",
    "PromptMode",
    "PromptTemplate",
    "TranscriptCompressionResult",
    "TranscriptAnalysis",
    "SDEncoderV2",
    "SDEncoder",
    "ThreadEncoder",
    "SysPromptEncoder",
]

__locals = locals()
for __name in __all__:
    if not __name.startswith("__"):
        setattr(__locals[__name], "__module__", "clm_core")



import re
from typing import Dict, List


class CLLMDecoder:
    def __init__(self):
        self.req_to_action = {
            "ANALYZE": "Analyze",
            "EXTRACT": "Extract",
            "GENERATE": "Generate",
            "SUMMARIZE": "Summarize",
            "TRANSFORM": "Convert",
            "EXPLAIN": "Explain",
            "COMPARE": "Compare",
            "CLASSIFY": "Classify",
            "DEBUG": "Debug",
            "OPTIMIZE": "Improve",
            "VALIDATE": "Validate",
            "SEARCH": "Search for",
            "RANK": "Rank",
            "PREDICT": "Predict",
            "FORMAT": "Format",
            "DETECT": "Detect",
            "CALCULATE": "Calculate",
            "AGGREGATE": "Aggregate",
            "DETERMINE": "Determine",
            "ROUTE": "Route",
            "EXECUTE": "Use",
            "LIST": "List",
            "QUERY": "What",
        }

        self.target_to_noun = {
            "CODE": "the code",
            "DATA": "the data",
            "DOCUMENT": "the document",
            "TRANSCRIPT": "the thread_encoder",
            "EMAIL": "the email",
            "TICKET": "the ticket",
            "REPORT": "the report",
            "CONCEPT": "the concept",
            "PROCEDURE": "the procedure",
            "ANSWER": "an answer",
            "ITEMS": "items",
            "CONTENT": "the content",
            "RESULT": "the result",
            "FEEDBACK": "feedback",
            "RESPONSE": "a response",
            "DESCRIPTION": "a description",
            "SUMMARY": "a summary",
            "PLAN": "a plan",
            "POST": "a post",
            "SYSTEM": "a system",
            "STRATEGY": "a strategy",
            "COMPLAINT": "a complaint",
            "METRICS": "the metrics",
            "ENDPOINT": "the API",
            "COMPONENT": "the component",
            "CONVERSATION": "the conversation",
            "RECORD": "the record",
            "PATTERN": "the pattern",
            "FEATURES": "the features",
            "LOGS": "the logs",
        }

        self.ctx_modifiers = {
            "TONE": {
                "PROFESSIONAL": "in a professional tone",
                "TECHNICAL": "in a technical tone",
                "CASUAL": "casually",
                "EMPATHETIC": "empathetically",
            },
            "STYLE": {
                "SIMPLE": "in simple terms",
                "DETAILED": "in detail",
                "CONCISE": "concisely",
            },
            "LENGTH": {
                "BRIEF": "briefly",
                "SHORT": "in short",
                "DETAILED": "in detail",
            },
        }

        self.out_formats = {
            "JSON": "in JSON format",
            "LIST": "as a list",
            "TABLE": "in a table",
            "MARKDOWN": "in markdown",
            "CSV": "as CSV",
        }

    def parse_token(self, token_str: str) -> Dict:
        """Parse CLLM token: [TYPE:VALUE:ATTR1=VAL1:ATTR2]"""
        parts = token_str.split(":")

        result = {
            "type": parts[0],
            "value": parts[1] if len(parts) > 1 else None,
            "attributes": {},
        }

        # Parse attributes (KEY=VALUE or standalone)
        for part in parts[2:]:
            if "=" in part:
                key, val = part.split("=", 1)
                result["attributes"][key] = val
            else:
                result["attributes"][part] = True

        return result

    def humanize_topic(self, topic: str) -> str:
        """
        Convert TOPIC to natural language
        THREE_PRIMARY_COLORS -> the three primary colors
        CAPITAL_OF_FRANCE -> the capital of France
        """
        # Replace underscores
        text = topic.replace("_", " ")

        # Lowercase
        text = text.lower()

        # Handle possessives
        text = text.replace("'s", "'s")

        # Add article if needed
        if not text.startswith(("the ", "a ", "an ")):
            # Add 'the' for most cases
            if not any(
                text.startswith(w)
                for w in ["what", "who", "where", "when", "why", "how"]
            ):
                text = "the " + text

        return text

    def format_question(self, topic: str, topic_type: str = None) -> str:
        """
        Format as a question
        topic='three primary colors' -> 'What are the three primary colors'
        topic='capital of france' -> 'What is the capital of France'
        """
        # Detect plural vs singular
        words = topic.split()
        is_plural = words and (
            words[0] in ["many", "few", "several", "some"]
            or (
                len(words) > 1
                and words[-1].endswith("s")
                and not words[-1].endswith("ss")
            )
        )

        # Choose "is" or "are"
        verb = "are" if is_plural else "is"

        # Format with proper article
        if topic.startswith(("the ", "a ", "an ")):
            return f"What {verb} {topic}"
        else:
            return f"What {verb} the {topic}"

    def combine_req_tokens(self, req_tokens: List[Dict]) -> str:
        """Intelligently combine multiple REQ tokens"""
        if not req_tokens:
            return ""

        if len(req_tokens) == 1:
            return self.req_to_action.get(
                req_tokens[0]["value"], req_tokens[0]["value"].lower()
            )

        reqs = [t["value"] for t in req_tokens]

        patterns = {
            ("GENERATE", "EXTRACT"): "Identify",
            ("TRANSFORM", "EXECUTE"): "Rewrite",
            ("TRANSFORM", "OPTIMIZE"): "Edit",
            ("GENERATE", "EXPLAIN"): "Generate",
            ("GENERATE", "OPTIMIZE"): "Generate",
            ("REQ:EXECUTE", "CALCULATE"): "Calculate",
            ("CLASSIFY", "GENERATE"): "Arrange",
            ("CLASSIFY", "EXECUTE"): "Classify",
            ("CLASSIFY", "OPTIMIZE"): "Classify",
            ("ANALYZE", "TRANSFORM"): "Analyze",
            ("GENERATE", "DEBUG"): "Construct",
            ("SUMMARIZE", "EXTRACT"): "Summarize",
        }

        req_tuple = tuple(reqs[:2])
        if req_tuple in patterns:
            return patterns[req_tuple]

        return self.req_to_action.get(reqs[0], reqs[0].lower())

    def decode(self, compressed: str) -> str:
        """
        Main decode function - converts CLLM tokens to natural language

        Args:
            compressed: Token string like "[REQ:GENERATE] [TARGET:ANSWER]"

        Returns:
            Natural language string
        """
        if not compressed or not compressed.strip():
            return ""

        token_strs = re.findall(r"\[([^\]]+)\]", compressed)
        if not token_strs:
            return compressed

        tokens = [self.parse_token(t) for t in token_strs]
        req_tokens = [t for t in tokens if t["type"] == "REQ"]
        target_tokens = [t for t in tokens if t["type"] == "TARGET"]
        extract_tokens = [t for t in tokens if t["type"] == "EXTRACT"]
        ctx_tokens = [t for t in tokens if t["type"] == "CTX"]
        out_tokens = [t for t in tokens if t["type"] == "OUT"]

        if not req_tokens and target_tokens:
            target = target_tokens[0]
            if "TYPE" in target["attributes"]:
                type_text = self.humanize_topic(target["attributes"]["TYPE"])
                return type_text.capitalize() + "."

            noun = self.target_to_noun.get(target["value"], target["value"].lower())
            return noun.capitalize() + "."

        if not req_tokens:
            return compressed

        action = self.combine_req_tokens(req_tokens)
        is_question = req_tokens[0]["value"] == "QUERY"

        if not target_tokens:
            return action + "."

        target = target_tokens[0]
        target_val = target["value"]

        if "TOPIC" in target["attributes"]:
            topic = target["attributes"]["TOPIC"]
            topic_text = self.humanize_topic(topic)

            if is_question:
                question = self.format_question(
                    topic_text, target["attributes"].get("TYPE")
                )
                return question + "?"
            else:
                sentence = f"{action} {topic_text}"

        elif "TYPE" in target["attributes"]:
            type_val = target["attributes"]["TYPE"]
            type_text = self.humanize_topic(type_val)

            if is_question:
                question = self.format_question(type_text)
                return question + "?"
            else:
                sentence = f"{action} {type_text}"

        elif "DOMAIN" in target["attributes"]:
            domain = target["attributes"]["DOMAIN"].lower()
            noun = self.target_to_noun.get(target_val, target_val.lower())
            noun = noun.replace("the ", "")
            sentence = f"{action} the {domain} {noun}"

        else:
            noun = self.target_to_noun.get(target_val, target_val.lower())

            if is_question:
                question = self.format_question(
                    noun.replace("the ", "").replace("a ", "").replace("an ", "")
                )
                return question + "?"
            else:
                sentence = f"{action} {noun}"

        if extract_tokens:
            fields = extract_tokens[0]["value"].split("+")
            field_text = ", ".join(f.replace("_", " ").lower() for f in fields)
            if len(fields) > 1:
                parts = field_text.rsplit(", ", 1)
                field_text = " and ".join(parts)
            sentence += f" and extract the {field_text}"

        for ctx in ctx_tokens:
            val = ctx["value"]
            if "=" in val:
                key, value = val.split("=", 1)
                if key in self.ctx_modifiers and value in self.ctx_modifiers[key]:
                    sentence += " " + self.ctx_modifiers[key][value]

        if out_tokens:
            fmt = out_tokens[0]["value"]
            if fmt in self.out_formats:
                sentence += " " + self.out_formats[fmt]

        if sentence and not sentence[0].isupper():
            sentence = sentence[0].upper() + sentence[1:]

        if not sentence.endswith((".", "?", "!")):
            sentence += "."

        return sentence

    def batch_decode(self, compressed_list: List[str]) -> List[str]:
        """Decode multiple prompts"""
        return [self.decode(c) for c in compressed_list]



from spacy.language import Language

from clm_core.utils.parser_rules import BaseRules
from clm_core.utils.vocabulary import BaseVocabulary
from ._configuration_prompt_encoder import ConfigurationPromptEncoder
from clm_core import SysPromptConfig

from ._schemas import PromptMode
from ._task_prompt_encoder import TaskPromptEncoder
from clm_core.types import CLMOutput

COMPONENT = "SYSTEM_PROMPT"


class SysPromptEncoder:
    def __init__(
        self,
        *,
        nlp: Language,
        config: SysPromptConfig = SysPromptConfig(),
        vocab: BaseVocabulary,
        rules: BaseRules,
    ):
        """
        Initialize encode
        """
        self._task_prompt = TaskPromptEncoder(
            nlp=nlp, config=config, rules=rules, vocab=vocab
        )
        self._configuration_prompt = ConfigurationPromptEncoder(
            nlp=nlp, vocab=vocab, rules=rules, config=config
        )

    def bind(self, out: CLMOutput, **kwargs: dict) -> str:
        return self._configuration_prompt.bind(out=out, **kwargs)

    @staticmethod
    def _detect_prompt_mode(text: str) -> PromptMode:
        """Detect which prompt we are dealing with.
        This is intentionally conservative to avoid hallucination
        """
        tl = text[:150].lower()

        if any(
            phrase in tl
            for phrase in (
                "you are an ai",
                "your role",
                "follow the rules",
                "capabilities:",
                "safety boundaries",
                "custom instructions",
                "<basic_rules>",
                "<custom_rules>",
            )
        ):
            return PromptMode.CONFIGURATION
        return PromptMode.TASK

    def compress(self, prompt: str, verbose: bool = False) -> CLMOutput:
        """
        Compress a natural language prompt into CLLM format

        Args:
            prompt: Natural language prompt to compress
            verbose: Print detailed compression steps

        Returns:
            CompressionResult with compressed format and metadata
        """
        mode = self._detect_prompt_mode(prompt)
        if mode == PromptMode.TASK:
            self._task_prompt.compress(prompt=prompt, verbose=verbose)
        return self._configuration_prompt.compress(prompt=prompt)

    def compress_batch(
        self, prompts: list[str], verbose: bool = False
    ) -> list[CLMOutput]:
        """Compress multiple prompts"""
        return [self.compress(prompt=prompt, verbose=verbose) for prompt in prompts]



from clm_core.utils.parser_rules import BaseRules


class FRRules(BaseRules):
    """French language pattern matching rules"""

    def __init__(self):
        super().__init__()

    @property
    def COMPARISON_MAP(self) -> dict[str, str]:
        return {
            r"\bdifférences?\b": "DIFFERENCES",
            r"\bdistinguer\b": "DIFFERENCES",
            r"\bcontraster\b": "DIFFERENCES",
            r"\bsimilarités?\b": "SIMILARITIES",
            r"\bressemblances?\b": "SIMILARITIES",
            r"\bcommun\b": "SIMILARITIES",
            r"\bavantages\s*(et|&)?\s*inconvénients\b": "PROS_CONS",
            r"\bpour\s*(et|&)?\s*contre\b": "PROS_CONS",
            r"\bpros\s*(et|&)?\s*cons\b": "PROS_CONS",
            r"\bcompromis\b": "TRADEOFFS",
            r"\btrade-?offs?\b": "TRADEOFFS",
        }

    @property
    def DOMAIN_REGEX(self) -> dict[str, str]:
        return {
            "SUPPORT": r"\b(appel|ticket|dossier|client|support|réclamation|helpdesk)\b",
            "TECHNICAL": r"\b(erreur|bug|crash|stacktrace|api|débogage|serveur|exception)\b",
            "DOCUMENT": r"\b(document|article|manuel|transcription|guide)\b",
            "BUSINESS": r"\b(rapport|exécutif|analyse|kpi|tableau de bord)\b",
            "LEGAL": r"\b(contrat|politique|conformité|clause|rgpd)\b",
            "FINANCE": r"\b(facture|facturation|paiement|remboursement|transaction)\b",
            "SECURITY": r"\b(violation|menace|risque|malware|audit)\b",
            "MEDICAL": r"\b(patient|clinique|diagnostic|traitement)\b",
            "SALES": r"\b(lead|crm|opportunité|prospect)\b",
            "EDUCATION": r"\b(leçon|enseignant|étudiant|programme)\b",
        }

    @property
    def DURATION_PATTERNS(self) -> list[str]:
        return [
            r"(\d+)[\s-]*(?:minute|min)s?",
            r"(\d+)[\s-]*(?:heure|hr)s?",
        ]

    @property
    def STANDARD_FIELD_KEYWORDS(self) -> dict[str, str]:
        return {
            r"\bproblème(s)?\b": "ISSUE",
            r"\bquestion(s)?\b": "PROBLEM",
            r"\berreur(s)?\b": "ERROR",
            r"\bbug(s)?\b": "BUG",
            r"\bnom(s)?\b": "NAMES",
            r"\bdate(s)?\b": "DATES",
            r"\bmontant(s)?\b": "AMOUNTS",
            r"\bsomme(s)?\b": "AMOUNTS",
            r"\bemail(s)?\b": "EMAILS",
            r"\bcourriel(s)?\b": "EMAILS",
            r"\btéléphone(s)?\b": "PHONES",
            r"\badresse(s)?\b": "ADDRESSES",
            r"\bsentiment\b": "SENTIMENT",
            r"\burgence\b": "URGENCY",
            r"\bpriorité\b": "PRIORITY",
            r"\bcatégorie\b": "CATEGORY",
            r"\baction(s)?\b": "ACTIONS",
            r"\bprochaines étapes\b": "NEXT_STEPS",
            r"\bétapes suivantes\b": "NEXT_STEPS",
            r"\béléments d'action\b": "ACTIONS",
            r"\bdate(s)? limite\b": "DEADLINES",
            r"\bdélai(s)?\b": "DEADLINES",
            r"\béchéance(s)?\b": "DEADLINES",
            r"\bsécurité\b": "SECURITY",
            r"\bperformance\b": "PERFORMANCE",
        }

    @property
    def AUDIENCE_MAP(self) -> dict[str, str]:
        return {
            r"\bnon[- ]?technique\b": "NON_TECHNICAL",
            r"\b(débutant|débutants|enfant|enfants|simple|général)\b": "BEGINNER",
            r"\b(expert|avancé|spécialiste|professionnels)\b": "EXPERT",
            r"\btechnique\b": "TECHNICAL",
            r"\b(affaires|exécutifs|direction|management)\b": "BUSINESS",
        }

    @property
    def LENGTH_MAP(self) -> dict[str, str]:
        return {
            r"\b(bref|brève|court|concis|rapide|résumé)\b": "BRIEF",
            r"\b(détaillé|complet|exhaustif|approfondi|étendu)\b": "DETAILED",
        }

    @property
    def STYLE_MAP(self) -> dict[str, str]:
        return {
            r"\b(simple|facile|clair)\b": "SIMPLE",
            r"\b(formel|professionnel|d'affaires)\b": "FORMAL",
        }

    @property
    def TONE_MAP(self) -> dict[str, str]:
        return {
            r"\bprofessionnel\b": "PROFESSIONAL",
            r"\bformel\b": "PROFESSIONAL",
            r"\bd'affaires\b": "PROFESSIONAL",
            r"\bdécontracté\b": "CASUAL",
            r"\binformel\b": "CASUAL",
            r"\bamical\b": "CASUAL",
            r"\bempathique\b": "EMPATHETIC",
            r"\bcompatissant\b": "EMPATHETIC",
            r"\bcompréhensif\b": "EMPATHETIC",
        }

    @property
    def NUMBER_WORDS(self) -> dict[str, int]:
        return {
            "un": 1,
            "une": 1,
            "deux": 2,
            "paire": 2,
            "couple": 2,
            "trois": 3,
            "quatre": 4,
            "cinq": 5,
            "six": 6,
            "sept": 7,
            "huit": 8,
            "neuf": 9,
            "dix": 10,
            "peu": -1,
            "quelques": -1,
            "plusieurs": -2,
            "beaucoup": -3,
        }

    @property
    def SPEC_PATTERNS(self) -> list[tuple[str, str]]:
        return [
            (r"(\d+)\s*lignes?\b", "LINES"),
            (r"(\d+)\s*mots?\b", "WORDS"),
            (r"(\d+)\s*(?:éléments?|choses?|items?)\b", "ITEMS"),
            (r"(\d+)\s*(?:conseils?|suggestions?)\b", "COUNT"),
            (r"(\d+)\s*(?:exemples?|instances?)\b", "COUNT"),
            (r"(\d+)\s*(?:étapes?|phases?)\b", "STEPS"),
            (r"(\d+)\s*(?:façons?|méthodes?|manières?)\b", "COUNT"),
        ]

    @property
    def PROGRAMMING_LANGUAGE_PATTERN(self) -> list[tuple[str, str]]:
        return [
            (r"\bpython\b", "PYTHON"),
            (r"\bdjango\b", "PYTHON"),
            (r"\bflask\b", "PYTHON"),
            (r"\bpandas\b", "PYTHON"),
            (r"\b\.py\b", "PYTHON"),
            (r"\bjavascript\b", "JAVASCRIPT"),
            (r"\bnode\.?js\b", "JAVASCRIPT"),
            (r"\breact\b", "JAVASCRIPT"),
            (r"\bvue\b", "JAVASCRIPT"),
            (r"\bangular\b", "JAVASCRIPT"),
            (r"\b\.js\b", "JAVASCRIPT"),
            (r"\bjava\b(?!script)", "JAVA"),
            (r"\bspring\b", "JAVA"),
            (r"\bmaven\b", "JAVA"),
            (r"\bc\+\+\b", "CPP"),
            (r"\bcpp\b", "CPP"),
            (r"\bgolang\b", "GO"),
            (r"\bgo\s+(?:code|programme|script)", "GO"),
            (r"\brust\b", "RUST"),
            (r"\btypescript\b", "TYPESCRIPT"),
            (r"\b\.ts\b", "TYPESCRIPT"),
        ]

    @property
    def EXTRACTION_INDICATORS(self) -> list[str]:
        return [
            r"\bextraire\b",
            r"\bidentifier\b",
            r"\btrouver\b",
            r"\bobtenir\b",
            r"\bretirer\b",
            r"\bsouligner\b",
            r"\bmontrer\b",
            r"\bretourner\b",
            r"\brécupérer\b",
            r"\bcomparer\b",
            r"\bcontraster\b",
            r"\blister\b",
            r"\bquels sont\b",
            r"\bqu'est-ce que\b",
        ]

    @property
    def QA_CRITERIA(self) -> dict[str, str]:
        return {
            r"\b(vérification|vérifier|vérifié)\b": "VERIFICATION",
            r"\b(politique|politiques|respect de la politique)\b": "POLICY",
            r"\b(compétences relationnelles|empathie|clarté|responsabilité)\b": "SOFT_SKILLS",
            r"\b(précision|précis|exactitude)\b": "ACCURACY",
            r"\b(conformité|conforme|violations?)\b": "COMPLIANCE",
            r"\b(sentiment|émotion|humeur)\b": "SENTIMENT",
            r"\b(divulgations?|divulgations obligatoires?)\b": "DISCLOSURES",
        }

    @property
    def QA_INDICATORS(self) -> list[str]:
        return [
            r"\bscore\b",
            r"\bnote\b",
            r"\bqa\b",
            r"\bassurance qualité\b",
            r"\bconformité\b",
            r"\baudit\b",
        ]

    @property
    def QUESTION_PATTERNS(self) -> list[tuple[str, int]]:
        return [
            (
                r"qu(?:'est-ce que|el|elle) (?:est|sont) (?:le |la |les |un |une )?([\w\s]+?)(?:\?|$)",
                1,
            ),
            (r"comment (?:fonctionne|fonctionnent|peut|peuvent) ([\w\s]+?)(?:\?|$)", 1),
            (r"pourquoi (?:est|sont|est-ce que) ([\w\s]+?)(?:\?|$)", 1),
            (r"où (?:est|sont|se trouve) ([\w\s]+?)(?:\?|$)", 1),
            (r"quand (?:est|était|sera) ([\w\s]+?)(?:\?|$)", 1),
            (r"qui (?:est|sont|était|étaient) (?:le |la )?([\w\s]+?)(?:\?|$)", 1),
        ]

    @property
    def EXPLAIN_PATTERNS(self) -> list[tuple[str, int]]:
        return [
            (r"(?:explique|décris) comment ([\w\s]+?)(?: fonctionne| marche|$)", 1),
            (
                r"(?:explique|décris|clarifie|détaille) (?:le |la |les |un |une )?([\w\s]+?)(?:\s+en|\s+avec|\s+pour|\.|\?|$)",
                1,
            ),
            (r"(?:dis-moi|parle-moi de) ([\w\s]+?)(?:\s+en|\s+avec|\.|\?|$)", 1),
        ]

    @property
    def CONCEPT_PATTERN(self) -> tuple[str, int]:
        return (
            r"(?:concept|idée|notion|principe|théorie) de ([\w\s]+?)(?:\s+en|\.|\?|$)",
            1,
        )

    @property
    def PROCEDURE_PATTERN(self) -> tuple[str, int]:
        return (
            r"comment (?:puis-je|peut-on|faire pour) ([\w\s]+?)(?:\s+en|\s+avec|\.|\?|$)",
            1,
        )

    @property
    def CLEANUP_TAIL(self) -> str:
        return r"\s+(de|en|pour|avec|sur|depuis|à|au|par|détail|détails|technique|spécifique)$"

    @property
    def SUBJECT_PATTERNS(self) -> list[tuple[str, str]]:
        return [
            (r"\bverbe[s]?\b", "VERB"),
            (r"\bnom[s]?\b", "NOUN"),
            (r"\badjectif[s]?\b", "ADJECTIVE"),
            (r"\badverbe[s]?\b", "ADVERB"),
            (r"\bpronom[s]?\b", "PRONOUN"),
            (r"\bpréposition[s]?\b", "PREPOSITION"),
            (r"\bconjonction[s]?\b", "CONJUNCTION"),
            (r"\bconseil[s]?\b", "TIP"),
            (r"\bsuggestion[s]?\b", "TIP"),
            (r"\bméthode[s]?\b", "METHOD"),
            (r"\btechnique[s]?\b", "TECHNIQUE"),
            (r"\bstratégie[s]?\b", "STRATEGY"),
            (r"\bapproche[s]?\b", "APPROACH"),
            (r"\bpratique[s]?\b", "PRACTICE"),
            (r"\balgorithme[s]?\b", "ALGORITHM"),
            (r"\bfonction[s]?\b", "FUNCTION"),
            (r"\bformule[s]?\b", "FORMULA"),
            (r"\béquation[s]?\b", "EQUATION"),
            (r"\bthéorème[s]?\b", "THEOREM"),
            (r"\bpreuve[s]?\b", "PROOF"),
            (r"\bexemple[s]?\b", "EXAMPLE"),
            (r"\bidée[s]?\b", "IDEA"),
            (r"\bfaçon[s]?\b", "METHOD"),
            (r"\bmanière[s]?\b", "METHOD"),
            (r"\bétape[s]?\b", "STEP"),
            (r"\bfacteur[s]?\b", "FACTOR"),
            (r"\braison[s]?\b", "REASON"),
            (r"\bmotif[s]?\b", "REASON"),
            (r"\bavantage[s]?\b", "BENEFIT"),
            (r"\binconvénient[s]?\b", "DISADVANTAGE"),
            (r"\bcaractéristique[s]?\b", "FEATURE"),
            (r"\bmétrique[s]?\b", "METRIC"),
            (r"\bindicateur[s]?\b", "INDICATOR"),
            (r"\bperspective[s]?\b", "INSIGHT"),
            (r"\bconstat[s]?\b", "FINDING"),
            (r"\bdécouverte[s]?\b", "FINDING"),
        ]

    @property
    def TYPE_MAP(self) -> dict[str, str]:
        return {
            "appel": "CALL",
            "appel téléphonique": "CALL",
            "réunion": "MEETING",
            "rencontre": "MEETING",
            "chat": "CHAT",
            "conversation": "CONVERSATION",
            "rapport": "REPORT",
            "article": "ARTICLE",
        }

    @property
    def CONTEXT_MAP(self) -> dict[str, str]:
        return {
            "client": "CUSTOMER",
            "support": "SUPPORT",
            "ventes": "SALES",
            "technique": "TECHNICAL",
        }

    @property
    def ISSUE_PATTERNS(self) -> list[str]:
        return [
            r"à propos de\s+([\w\s]+?)(?:\s+et|$)",
            r"concernant\s+([\w\s]+?)(?:\s+et|$)",
            r"au sujet de\s+([\w\s]+?)(?:\s+et|$)",
            r"relatif à\s+([\w\s]+?)(?:\s+et|$)",
        ]



import json
import re
from enum import Enum
from typing import Optional, Self, Literal, Annotated, TypeAlias, Union

import spacy
from pydantic import (
    BaseModel,
    Field,
    PrivateAttr,
    computed_field,
    ConfigDict,
    field_validator,
    field_serializer,
    model_validator,
)

from clm_core.utils.parser_rules import BaseRules
from clm_core.utils.vocabulary import BaseVocabulary
from clm_core.dictionary import rules_map, vocab_map, patterns_map

ORIGINAL_INPUT: TypeAlias = Union[str, dict, list]
LANG: TypeAlias = Literal["en", "fr", "es", "pt"]


class CLMOutput(BaseModel):
    original: ORIGINAL_INPUT = Field(
        ..., description="A generic original input. It can be a str, list, or dict"
    )
    component: str = Field(
        ..., description="Component's name (i.e. Transcript, SD, System Prompt)"
    )
    compressed: str = Field(..., description="Compressed output.")
    metadata: dict = Field(
        ...,
        description="Metadata of the compressing input. It can include specific things from each component",
    )

    @model_validator(mode="after")
    def validate_compression_ratio(self) -> Self:
        """If compression ratio is negative (expanded), use original instead."""
        if self.c_tokens > self.n_tokens:
            original = self.original
            if isinstance(original, str):
                self.compressed = original
            else:
                self.compressed = json.dumps(original, ensure_ascii=False)
            self.metadata["description"] = (
                "CL Tokens greater than NL token. Keeping NL input"
            )
        return self

    @field_validator("compressed", mode="before")
    @classmethod
    def validate_compressed(cls, c: str) -> str:
        """Normalize whitespace: collapse all whitespace (tabs, newlines, spaces) to single spaces."""
        return re.sub(r"\s+", " ", c).strip()

    @staticmethod
    def _estimate_tokens(data: str | dict | list) -> int:
        """Estimate token count (~4 chars per token)."""
        if isinstance(data, str):
            text = data
        else:
            text = json.dumps(data, ensure_ascii=False)
        return max(1, len(text) // 4)

    @computed_field
    @property
    def n_tokens(self) -> int:
        """Estimated input token count."""
        return self._estimate_tokens(self.original)

    @computed_field
    @property
    def c_tokens(self) -> int:
        """Estimated compressed token count."""
        return self._estimate_tokens(self.compressed)

    @computed_field
    @property
    def compression_ratio(self) -> float:
        """Compression ratio based on token reduction."""
        if self.n_tokens == 0:
            return 0.0
        return round((1 - self.c_tokens / self.n_tokens) * 100, 1)

    def to_dict(self) -> dict:
        raise NotImplementedError


class FieldImportance(Enum):
    """Field importance levels

    CRITICAL: Always include (id, name)
    HIGH: Usually include (description, category)
    MEDIUM: Sometimes include (tags, metadata)
    LOW: Rarely include (timestamps, internal codes)
    NEVER: Never include (passwords, secrets)
    """

    CRITICAL = 1.0
    HIGH = 0.8
    MEDIUM = 0.5
    LOW = 0.2
    NEVER = 0.0


class SDCompressionConfig(BaseModel):
    required_fields: Optional[list[str]] = Field(
        default=None, description="Always include these"
    )
    auto_detect: Optional[bool] = Field(
        default=True, description="Approach 2: Auto-detect rule"
    )
    drop_non_required_fields: Optional[bool] = Field(
        default=True, description="Whether or not to drop no required fields"
    )
    importance_threshold: Optional[float] = Field(
        default=0.5, description="Include fields above this threshold"
    )
    field_importance: Optional[dict[str, float]] = Field(
        default=None,
        description="Custom importance scores. Overrides default thresholds.",
    )
    excluded_fields: Optional[list[str]] = Field(
        default=None, description="Never include these"
    )
    max_truncation_length: Optional[int] = Field(
        default=200,
        description="Truncate long text. If max config truncation mapping not defined, will use this for all",
    )
    max_truncation_mapping: Optional[dict[str, int]] = Field(
        default=None,
        description="Truncation mapping. This overrides max_truncation_mapping and truncate each field individually",
    )
    preserve_structure: Optional[bool] = Field(
        default=True, description="Keep nested dicts/lists"
    )
    default_fields_importance: dict[str, FieldImportance] = Field(
        frozen=True,
        default_factory=lambda: {
            "id": FieldImportance.CRITICAL,
            "uuid": FieldImportance.CRITICAL,
            "external_id": FieldImportance.CRITICAL,
            "name": FieldImportance.HIGH,
            "title": FieldImportance.HIGH,
            "type": FieldImportance.HIGH,
            "category": FieldImportance.HIGH,
            "subcategory": FieldImportance.MEDIUM,
            "tags": FieldImportance.HIGH,
            "description": FieldImportance.HIGH,
            "details": FieldImportance.MEDIUM,
            "notes": FieldImportance.LOW,
            "status": FieldImportance.CRITICAL,
            "priority": FieldImportance.HIGH,
            "severity": FieldImportance.HIGH,
            "resolution": FieldImportance.HIGH,
            "owner": FieldImportance.HIGH,
            "assignee": FieldImportance.MEDIUM,
            "department": FieldImportance.MEDIUM,
            "channel": FieldImportance.HIGH,
            "language": FieldImportance.MEDIUM,
            "source": FieldImportance.LOW,
            "metadata": FieldImportance.LOW,
            "created_at": FieldImportance.LOW,
            "updated_at": FieldImportance.LOW,
            "version": FieldImportance.LOW,
        },
        description="Importance scores for default fields. Overrides default thresholds.",
    )

    @computed_field
    @property
    def simple_fields(self) -> tuple[str, ...]:
        """Default fields that are mapped if no field is configured"""
        return (
            "id",
            "uuid",
            "title",
            "name",
            "type",
            "priority",
            "email",
            "article_id",
            "product_id",
        )

    @computed_field
    @property
    def default_fields_order(self) -> tuple[str, ...]:
        """
        Order for default fields. IDs take precedence over other fields. "
        "If a field is not in this list, it will be placed at the end.
        """
        return (
            "id",
            "uuid",
            "priority",
            "article_id",
            "product_id",
            "title",
            "name",
            "type",
        )

    @field_validator("default_fields_importance", mode="before")
    @classmethod
    def convert_float_to_field_importance(cls, v: dict) -> dict[str, FieldImportance]:
        if not isinstance(v, dict):
            return v
        result = {}
        for key, value in v.items():
            if isinstance(value, (int, float)):
                for member in FieldImportance:
                    if member.value == value:
                        result[key] = member
                        break
                else:
                    raise ValueError(f"No FieldImportance enum matches value {value}")
            else:
                result[key] = value
        return result

    @field_serializer("default_fields_importance")
    @classmethod
    def serialize_field_importance(
        cls, v: dict[str, FieldImportance]
    ) -> dict[str, float]:
        return {key: importance.value for key, importance in v.items()}

    model_config = ConfigDict(use_enum_values=False)


class SysPromptConfig(BaseModel):
    lang: str = Field(default="en", description="Language of the prompt")
    infer_types: Optional[bool] = Field(
        default=False, description="Infer types for output fields"
    )
    use_structured_output_abstraction: Optional[bool] = Field(
        default=True,
        description="If to compress output structure define with CL or keep it as-is",
    )
    add_examples: Optional[bool] = Field(
        default=False,
        description="Add examples based on extracted ones from input if exist",
    )
    add_attrs: Optional[bool] = Field(
        default=True,
        description="Add extra attributes from input prompt. "
        "This can be specifications found in prompt, enums/constraints values defined",
    )


class CLMConfig(BaseModel):
    lang: Annotated[LANG, Field(default="en", description="Language of the model")]
    ds_config: SDCompressionConfig = Field(
        default_factory=lambda: SDCompressionConfig(),
        description="Configuration for data Structure Data compression",
    )
    sys_prompt_config: SysPromptConfig = Field(
        default_factory=lambda: SysPromptConfig(),
        description="Configuration for system prompt",
    )
    redaction_pattern: str = Field(
        default=r"\[\*+REDACTED\*+\]|\*{3,}|\[REDACTED\]|<redacted>|XXX+|\[PII\]",
        description="Regex pattern to detect redacted fields in transcripts",
    )
    _nlp_cache: Optional[spacy.Language] = PrivateAttr(default=None)

    @computed_field
    @property
    def vocab(self) -> BaseVocabulary:
        return vocab_map[self.lang]

    @computed_field
    @property
    def nlp_model(self) -> spacy.Language:
        """
        Load spaCy model for the configured language.
        Cached at instance level to avoid repeated loading.
        """
        if self._nlp_cache is not None:
            return self._nlp_cache
        model_map = {
            "en": "en_core_web_sm",
            "es": "es_core_news_sm",
            "pt": "pt_core_news_sm",
            "fr": "fr_core_news_sm",
        }
        model_name = model_map.get(self.lang)
        if model_name is None:
            raise NotImplementedError(
                f"Model for language {self.lang} not supported yet"
            )
        self._nlp_cache = spacy.load(model_name)
        return self._nlp_cache

    @computed_field
    @property
    def rules(self) -> BaseRules:
        return rules_map[self.lang]

    @computed_field(return_type=object)
    @property
    def patterns(self):
        return patterns_map[self.lang]

    model_config = ConfigDict(
        arbitrary_types_allowed=True,
    )



from spacy.tokens import Doc
from clm_core.components.sys_prompt import Target

from ._target.target_normalizer import TargetNormalizer
from ._target.extractors import (
    ImperativeExtractor,
    QuestionExtractor,
    NounExtractor,
    CompoundExtractor,
    PatternExtractor,
    FallbackExtractor,
)
from ._target.attributes import AttributeEnhancer
from ..utils.parser_rules import BaseRules
from ..utils.vocabulary import BaseVocabulary


class TargetExtractor:
    """
    Main target extraction orchestrator

    Coordinates multiple extraction strategies in priority order
    """

    def __init__(self, nlp, vocab: BaseVocabulary, rules: BaseRules):
        self.nlp = nlp
        self.vocab = vocab
        self.rules = rules

        self.imperative_extractor = ImperativeExtractor(
            nlp=self.nlp,
            vocab=self.vocab,
            rules=self.rules,
        )
        self.question_extractor = QuestionExtractor(
            nlp=self.nlp,
            vocab=self.vocab,
            rules=self.rules,
        )
        self.noun_extractor = NounExtractor(
            nlp=self.nlp, vocab=self.vocab, rules=self.rules
        )
        self.compound_extractor = CompoundExtractor(
            nlp=self.nlp, vocab=self.vocab, rules=self.rules
        )
        self.pattern_extractor = PatternExtractor(
            nlp=self.nlp, vocab=self.vocab, rules=self.rules
        )
        self.fallback_extractor = FallbackExtractor(
            nlp=self.nlp, vocab=self.vocab, rules=self.rules
        )
        self.attribute_enhancer = AttributeEnhancer(
            nlp=self.nlp, vocab=self.vocab, rules=self.rules
        )
        self.normalizer = TargetNormalizer()

    def extract(
        self, text: str, detected_req_tokens: list[str] | None = None, doc: Doc = None
    ) -> Target:
        """Main extraction pipeline

        Priority order:
        1. Imperative patterns (highest confidence)
        2. Question patterns
        3. Noun matches
        4. Compound phrases
        5. Pattern detection
        6. Fallback

        Args:
            text (str): The input text to extract targets from.
            detected_req_tokens (list[str] | None): Optional list of detected requirement tokens.
            doc (Doc): Optional pre-processed spaCy doc to reuse.

        Returns:
            list[Target]: A list of extracted targets.

        Example:
            >>> extractor = TargetExtractor(nlp, vocab)
            >>> text = "I need a laptop with a 16GB RAM and a 512GB SSD."
            >>> detected_req_tokens = ["laptop", "ram", "ssd"]
            >>> targets = extractor.extract(text, detected_req_tokens)
            >>> targets
            [Target(text='laptop', attributes={'ram': '16GB', 'ssd': '512GB'})]
        """
        if doc is None:
            doc = self.nlp(text)

        target = self.imperative_extractor.extract(text, detected_req_tokens, doc)
        if target:
            return target

        target = self.question_extractor.extract(text, doc)
        if target:
            return target

        targets = []

        targets.extend(self.noun_extractor.extract(text, doc))
        targets.extend(self.compound_extractor.extract(text, doc))
        targets.extend(self.pattern_extractor.extract(text, doc))

        if not targets:
            target = self.fallback_extractor.extract(text, detected_req_tokens, doc)
            if target:
                targets.append(target)

        return self.normalizer.normalize_many(targets)
